from quick_resto_API.quick_resto_objects.modules.front.table_scheme import TableScheme
from quick_resto_API.quick_resto_objects.modules.front.device_command import DeviceCommand
from quick_resto_API.quick_resto_objects.quick_resto_object import QuickRestoObject


class VirtualPosTerminal(QuickRestoObject):
    @property
    def device_id(self) -> str:
        return self._device_id

    @property
    def manufacturer(self) -> str:
        return self._manufacturer

    @property
    def model(self) -> str:
        return self._model

    @property
    def connected(self) -> bool:
        return self._connected

    @property
    def name(self) -> str:
        return self._name

    @property
    def deleted(self) -> bool:
        return self._deleted

    @property
    def activate_on_current_terminal(self) -> bool:
        return self._activate_on_current_terminal

    @property
    def state(self) -> str:
        return self._state

    @property
    def table_scheme(self) -> TableScheme:
        return self._table_scheme

    @property
    def command(self) -> DeviceCommand:
        return self._command

    @property
    def business(self) -> dict:
        return self._business

    @property
    def connection_kind(self) -> str:
        return self._connection_kind

    @property
    def pos_additional_info(self) -> str:
        return self._pos_additional_info

    @property
    def arcus2_protocol(self) -> str:
        return self._arcus2_protocol

    @property
    def pos_printing_type(self) -> str:
        return self._pos_printing_type

    def __init__(self, deviceId: str = None, manufacturer: str = None, model: str = None, connected: bool = None,
                 name: str = None,
                 deleted: bool = None, activateOnCurrentTerminal: bool = None, state: str = None,
                 tableScheme: dict = None, command: dict = None,
                 business: dict = None, connectionKind: str = None, posAdditionalInfo: str = None,
                 arcus2Protocol: str = None,
                 posPrintingType: str = None, **kwargs):
        class_name = "ru.edgex.quickresto.modules.front.terminals.pos.VirtualPosTerminal"

        super().__init__(class_name=class_name, **kwargs)

        self._device_id: str = deviceId
        self._manufacturer: str = manufacturer
        self._model: str = model
        self._connected: bool = connected
        self._name: str = name
        self._deleted: bool = deleted
        self._activate_on_current_terminal: bool = activateOnCurrentTerminal
        self._state: str = state

        if tableScheme is not None: 
            self._table_scheme = TableScheme(**tableScheme)
        else:
            self._table_scheme = None

        if command is not None: 
            self._command = DeviceCommand(**command)
        else:
            self._command = None

        self._business: dict = business
        self._connection_kind: str = connectionKind
        self._pos_additional_info: str = posAdditionalInfo
        self._arcus2_protocol: str = arcus2Protocol
        self._pos_printing_type: str = posPrintingType



from quick_resto_API.quick_resto_objects.quick_resto_object import QuickRestoObject


class AccountBalance(QuickRestoObject):
    @property
    def available(self) -> float:
        return self._available

    @property
    def credit_hold(self) -> float:
        return self._credit_hold

    @property
    def debit_hold(self) -> float:
        return self._debit_hold

    @property
    def ledger(self) -> float:
        return self._ledger

    def __init__(self, available: float = None, creditHold: float = None, debitHold: float = None, ledger: float = None,
                 **kwargs):
        class_name = 'ru.edgex .quickresto.modules.crm.accounting.account.balance.AccountBalance'

        super().__init__(id=0, class_name=class_name, **kwargs)
        self._available: float = available
        self._credit_hold: float = creditHold
        self._debit_hold: float = debitHold
        self._ledger: float = ledger



from quick_resto_API.quick_resto_objects.quick_resto_object import QuickRestoObject


class Businessman(QuickRestoObject):
    @property
    def short_name(self) -> str:
        return self._short_name

    @property
    def egais_status(self) -> str:
        return self._egais_status

    @property
    def egais_activity_status(self) -> str:
        return self._egais_activity_status

    @property
    def first_name(self) -> str:
        return self._first_name

    @property
    def middle_name(self) -> str:
        return self._middle_name

    @property
    def last_name(self) -> str:
        return self._last_name

    def __init__(self, shortName: str = None, egaisStatus: str = None, egaisActivityStatus: str = None,
                 firstName: str = None,
                 middleName: str = None, lastName: str = None, **kwargs):
        class_name = "ru.edgex.quickresto.modules.warehouse.providers.Businessman"

        super().__init__(class_name=class_name, **kwargs)

        self._short_name: str = shortName
        self._egais_status: str = egaisStatus
        self._egais_activity_status: str = egaisActivityStatus
        self._first_name: str = firstName
        self._middle_name: str = middleName
        self._last_name: str = lastName



from enum import Enum

from quick_resto_API.quick_resto_objects.quick_resto_object import QuickRestoObject


class DeviceState(Enum):
    ACTIVE = "ACTIVE"
    INACTIVE = "INACTIVE"
    NONE = "NONE"


StrToDeviceState = {
    DeviceState.ACTIVE.value: DeviceState.ACTIVE,
    DeviceState.INACTIVE.value: DeviceState.INACTIVE,
}


def convert_str_to_device_state(value: str) -> DeviceState:
    if value in StrToDeviceState.keys():
        return StrToDeviceState[value]

    return DeviceState.NONE


class DeviceCommand(QuickRestoObject):
    @property
    def version(self) -> int:
        return self._version

    @property
    def state(self) -> DeviceState:
        return self._state

    def __init__(self, version: int = None, state: str = None, **kwargs):
        class_name = ""
        super().__init__(class_name=class_name, **kwargs)

        self._version: int = version
        self._state = convert_str_to_device_state(state)



from enum import Enum

from quick_resto_API.quick_resto_objects.modules.core.measure_unit import MeasureUnit
from quick_resto_API.quick_resto_objects.modules.core.store_item_tag import StoreItemTag
from quick_resto_API.quick_resto_objects.modules.warehouse.dish_sale import DishSale
from quick_resto_API.quick_resto_objects.quick_resto_object import QuickRestoObject


class SellingType(Enum):
    WHOLE = "WHOLE"
    PART = "PART"
    WEIGHT = "WEIGHT"
    NONE = "NONE"


StrToSellingType = {
    SellingType.WHOLE.value: SellingType.WHOLE,
    SellingType.PART.value: SellingType.PART,
    SellingType.WEIGHT.value: SellingType.WEIGHT
}


def convert_str_to_selling_type(value: str) -> SellingType:
    if value in StrToSellingType.keys():
        return StrToSellingType[value]

    return SellingType.NONE


class DishCategory(QuickRestoObject):
    @property
    def title(self) -> str:
        return self._title

    @property
    def name(self) -> str:
        return self._name

    @property
    def color(self) -> str:
        return self._color

    @property
    def display_on_terminal(self) -> bool:
        return self._display_on_terminal

    @property
    def measure_unit(self) -> MeasureUnit:
        return self._measure_unit

    @property
    def selling_type(self) -> SellingType:
        return self._selling_type

    @property
    def dish_sales(self) -> list:
        return self._dish_sales

    @property
    def store_item_tag(self) -> StoreItemTag:
        return self._store_item_tag

    def __init__(self, itemTitle: str = None, name: str = None, displayOnTerminal: bool = None, color: str = None,
                 sellingType: str = None,
                 measureUnit: dict = None, dishSales=None, storeItemTag=None, **kwargs):
        class_name: str = "ru.edgex.quickresto.modules.warehouse.nomenclature.dish.DishCategory"

        super().__init__(class_name=class_name, **kwargs)
        self._title: str = itemTitle
        self._name: str = name
        self._color: str = color
        self._display_on_terminal: bool = displayOnTerminal

        if measureUnit is not None: 
            self._measure_unit: MeasureUnit = MeasureUnit(**measureUnit)
        else:
            self._measure_unit = None

        self._selling_type: SellingType = convert_str_to_selling_type(sellingType)

        if storeItemTag is not None:
            self._store_item_tag = StoreItemTag(**storeItemTag)
        else:
            self._store_item_tag = None

        if dishSales is not None:
            self._dish_sales: list = [DishSale(**dish_sale) for dish_sale in dishSales]
        else:
            self._dish_sales = None



from enum import Enum

from quick_resto_API.quick_resto_objects.quick_resto_object import QuickRestoObject


class ContactMethodType(Enum):
    NONE = 'NONE'
    EMAIL = 'eMail'
    PHONE = 'phoneNumber'


StrToContactMethodType = {
    ContactMethodType.EMAIL.value: ContactMethodType.EMAIL,
    ContactMethodType.PHONE.value: ContactMethodType.PHONE
}


def convert_str_to_contact_method_type(method_type: str) -> ContactMethodType:
    if method_type in StrToContactMethodType.keys():
        return StrToContactMethodType[method_type]

    return ContactMethodType.NONE


class ContactMethod(QuickRestoObject):
    @property
    def value(self) -> str:
        return self._value

    @property
    def contact_type(self) -> ContactMethodType:
        return self._contact_type

    def __init__(self, type: str = None, value: str = None, **kwargs):
        class_name: str = 'modules.crm.accounting.account.contact_method'

        super().__init__(id=0, class_name=class_name, **kwargs)
        self._value: str = value
        self._contact_type: ContactMethodType = convert_str_to_contact_method_type(type)



from quick_resto_API.quick_resto_objects.modules.core.store_item_tag import StoreItemTag
from quick_resto_API.quick_resto_objects.modules.crm.fixed_discount import convert_str_to_type_discount, TypeDiscount
from quick_resto_API.quick_resto_objects.modules.warehouse.dish import Dish
from quick_resto_API.quick_resto_objects.modules.warehouse.dish_category import DishCategory
from quick_resto_API.quick_resto_objects.modules.warehouse.modifier_group import ModifierGroup
from quick_resto_API.quick_resto_objects.modules.warehouse.sale_place import SalePlace
from quick_resto_API.quick_resto_objects.quick_resto_object import QuickRestoObject


class TimeRange(QuickRestoObject):
    @property
    def start(self) -> str:
        return self._start

    @property
    def end(self) -> str:
        return self._end

    def __init__(self, start: str, end: str):
        self._start: str = start
        self._end: str = end


class Day(QuickRestoObject):
    @property
    def day(self) -> str:
        return self._day

    @property
    def active(self) -> bool:
        return self._active

    @property
    def time_range(self) -> TimeRange:
        return self._time_range

    def __init__(self, day: str, active: bool, timeRange: dict, **kwargs):
        class_name = "ru.edgex.quickresto.modules.crm.settings.markup.Day"
        super().__init__(class_name=class_name, **kwargs)

        self._day: str = day
        self._active: bool = active
        
        if timeRange is not None:
            self._time_range: TimeRange = TimeRange(**timeRange)
        else:
            self._time_range = None


class Markup(QuickRestoObject):
    @property
    def name(self) -> str:
        return self._name

    @property
    def deleted(self) -> bool:
        return self._deleted

    @property
    def type_discount(self) -> TypeDiscount:
        return self._type_discount

    @property
    def value(self) -> float:
        return self._value

    @property
    def operator_cancellable(self) -> bool:
        return self._operator_cancellable

    @property
    def date_range(self) -> TimeRange:
        return self._date_range

    @property
    def categories(self) -> list:
        return self._categories

    @property
    def dishes(self) -> list:
        return self._dishes

    @property
    def tags(self) -> list:
        return self._tags

    @property
    def modifier_groups(self) -> list:
        return self._modifier_groups

    @property
    def sale_places(self) -> list:
        return self._sale_places

    @property
    def days(self) -> list:
        return self._days

    def __init__(self, name: str = None, deleted: bool = None, typeDiscount: str = None, value: float = None,
                 operatorCancellable: bool = None,
                 dateRange: dict = None, categories: list = None, dishes: list = None, tags: list = None,
                 modifierGroups: list = None, salePlaces: list = None,
                 days: list = None, **kwargs):
        class_name = "ru.edgex.quickresto.modules.crm.settings.markup.Markup"

        super().__init__(class_name=class_name, **kwargs)

        self._name: str = name
        self._deleted: bool = deleted
        self._type_discount = convert_str_to_type_discount(typeDiscount)
        self._value: float = value
        self._operator_cancellable: bool = operatorCancellable

        if dateRange is not None: 
            self._date_range = TimeRange(**dateRange)
        else:
            self._date_range = None

        if categories is not None: 
            self._categories: list = [DishCategory(**category) for category in categories]
        else:
            self._categories = None

        if dishes is not None: 
            self._dishes: list = [Dish(**dish) for dish in dishes]
        else:
            self._dishes = None

        if tags is not None: 
            self._tags: list = [StoreItemTag(**tag) for tag in tags]
        else:
            self._tags = None

        if modifierGroups is not None: 
            self._modifier_groups: list = [ModifierGroup(**group) for group in modifierGroups]
        else:
            self._modifier_groups = None

        if salePlaces is not None: 
            self._sale_places: list = [SalePlace(**place) for place in salePlaces]
        else:
            self._sale_places = None

        if days is not None: 
            self._days: list = [Day(**day) for day in days]
        else:
            self._days = None



from quick_resto_API.quick_resto_objects.quick_resto_object import QuickRestoObject


class StoreItemTag(QuickRestoObject):
    @property
    def deleted(self) -> bool:
        return self._deleted

    @property
    def name(self) -> str:
        return self._name

    def __init__(self, name: str = None, deleted: bool = None, **kwargs):
        class_name = "ru.edgex.quickresto.modules.core.dictionaries.storeitemtag.StoreItemTag"

        super().__init__(class_name=class_name, **kwargs)

        self._deleted = deleted
        self._name = name



from quick_resto_API.quick_resto_objects.modules.front.hall import Hall
from quick_resto_API.quick_resto_objects.modules.front.table import Table
from quick_resto_API.quick_resto_objects.quick_resto_object import QuickRestoObject


class TableScheme(QuickRestoObject):
    @property
    def current_load(self) -> int:
        return self._current_load

    @property
    def deleted(self) -> bool:
        return self._deleted

    @property
    def width(self) -> int:
        return self._width

    @property
    def height(self) -> int:
        return self._height

    @property
    def title(self) -> str:
        return self._title

    @property
    def max_capacity(self) -> int:
        return self._max_capacity

    @property
    def name(self) -> str:
        return self._name

    @property
    def reservations(self) -> list:
        return self._reservations

    @property
    def tables(self) -> list:
        return self._tables

    @property
    def halls(self) -> list:
        return self._halls

    def __init__(self, currentLoad: int = None, deleted: bool = None, width: int = None, height: int = None,
                 itemTitle: str = None, maxCapacity: int = None,
                 name: str = None, tables: list = None, reservations: list = None, halls: list = None, **kwargs):
        class_name = "ru.edgex.quickresto.modules.front.tablemanagement.TableScheme"

        super().__init__(class_name=class_name, **kwargs)
        self._current_load: int = currentLoad
        self._deleted: bool = deleted
        self._width: int = width
        self._height: int = height
        self._title: str = itemTitle
        self._max_capacity: int = maxCapacity
        self._name: str = name
        self._reservations: list = reservations

        if tables is not None: 
            self._tables: list = [Table(**value) for value in tables]
        else:
            self._tables = None

        if halls is not None: 
            self._halls: list = [Hall(**value) for value in halls]
        else:
            self._halls = None



from enum import Enum
from quick_resto_API.quick_resto_objects.modules.front.kkm_terminal import KkmTerminal

from quick_resto_API.quick_resto_objects.modules.front.shift import Shift
from quick_resto_API.quick_resto_objects.modules.personnel.employee import Employee
from quick_resto_API.quick_resto_objects.modules.warehouse.sale_place import SalePlace
from quick_resto_API.quick_resto_objects.quick_resto_object import QuickRestoObject


class EncashmentType(Enum):
    CASHIN = "CASHIN"
    CASHOUT = "CASHOUT"
    NONE = "NONE"


StrToEncashmentType = {
    EncashmentType.CASHIN.value: EncashmentType.CASHIN,
    EncashmentType.CASHOUT.value: EncashmentType.CASHOUT,
}


def convert_str_to__encashment_Type(value: str) -> EncashmentType:
    if value in StrToEncashmentType.keys():
        return StrToEncashmentType[value]

    return EncashmentType.NONE


class Encashment(QuickRestoObject):
    @property
    def time_encashment(self) -> str:
        return self._time_encashment

    @property
    def amount(self) -> int:
        return self._amount

    @property
    def number(self) -> int:
        return self._number

    @property
    def boolean(self) -> bool:
        return self._boolean

    @property
    def employee(self) -> Employee:
        return self._employee

    @property
    def kkm(self) -> KkmTerminal:
        return self._kkm

    @property
    def sale_place(self) -> SalePlace:
        return self._sale_place

    @property
    def shift(self) -> Shift:
        return self._shift

    @property
    def type_transaction(self) -> EncashmentType:
        return self._type_transaction

    def __init__(self, timeEncashment: str = None, amount: int = None, number: int = None, boolean: bool = None,
                 employee: dict = None, kkm: dict = None, salePlace: dict = None,
                 shift: dict = None, typeTransaction: dict = None, **kwargs):
        class_name = "ru.edgex.quickresto.modules.front.encashment.Encashment"

        super().__init__(class_name=class_name, **kwargs)

        self._time_encashment: str = timeEncashment
        self._amount: int = amount
        self._number: int = number
        self._boolean: bool = boolean

        if employee is not None: 
            self._employee = Employee(**employee)
        else:
            self._employee = None

        if kkm is not None: 
            self._kkm = KkmTerminal(**kkm)
        else:
            self._kkm = None

        if salePlace is not None: 
            self._sale_place = SalePlace(**salePlace)
        else:
            self._sale_place = None

        if shift is not None: 
            self._shift = Shift(**shift)
        else:
            self._shift = None

        self._type_transaction = convert_str_to__encashment_Type(typeTransaction)



from quick_resto_API.quick_resto_objects.modules.core.measure_unit import MeasureUnit
from quick_resto_API.quick_resto_objects.modules.core.store_item_tag import StoreItemTag
from quick_resto_API.quick_resto_objects.modules.warehouse.dish_category import convert_str_to_selling_type, DishCategory, SellingType
from quick_resto_API.quick_resto_objects.modules.warehouse.dish_sale import DishSale
from quick_resto_API.quick_resto_objects.quick_resto_object import QuickRestoObject


class Dish(QuickRestoObject):
    @property
    def article(self) -> int:
        return self._article

    @property
    def base_price_in_list(self) -> float:
        return self._base_price_in_list

    @property
    def dish_sales(self) -> list:
        return self._dish_sales

    @property
    def exclude_discount(self) -> int:
        return self._exclude_discount

    @property
    def display_on_terminal(self) -> bool:
        return self._display_on_terminal

    @property
    def exclude_markup(self) -> int:
        return self._exclude_markup

    @property
    def measure_unit(self) -> MeasureUnit:
        return self._measure_unit

    @property
    def minimal_price(self) -> float:
        return self._minimal_price

    @property
    def name(self) -> str:
        return self._name

    @property
    def pack(self) -> float:
        return self._pack

    @property
    def price(self) -> float:
        return self._price

    @property
    def ratio(self) -> float:
        return self._ratio

    @property
    def title(self) -> str:
        return self._title

    @property
    def recipe(self) -> str:
        return self._recipe

    @property
    def selling_type(self) -> SellingType:
        return self._selling_type

    @property
    def store_quantity_kg(self) -> float:
        return self._store_quantity_kg

    @property
    def parent_id(self) -> int:
        return self._parent_id

    @property
    def parent_item(self) -> DishCategory:
        return self._parent_item

    @property
    def store_item_tag(self) -> StoreItemTag:
        return self._store_item_tag

    def __init__(self, article: str = None, basePriceInList: float = None, dishSales: list = None,
                 displayOnTerminal: bool = None,
                 excludeDiscount: bool = None, excludeMarkup: bool = None, measureUnit: dict = None,
                 minimalPrice: float = None, name: str = None,
                 pack: float = None, ratio: float = None, recipe: str = None, sellingType: str = None,
                 storeQuantityKg: float = None, parentId: int = None,
                 parentItem: dict = None, price: float = 0.0, itemTitle: str = "", storeItemTag=None, **kwargs):
        class_name: str = "ru.edgex.quickresto.modules.warehouse.nomenclature.dish.Dish"

        super().__init__(class_name=class_name, **kwargs)
        if (article != None): self._article: int = int(article)
        self._base_price_in_list: float = basePriceInList

        if dishSales is not None:
            self._dish_sales: list = [DishSale(**dish_sale) for dish_sale in dishSales]
        else:
            self._dish_sales = None

        self._display_on_terminal: bool = displayOnTerminal
        self._exclude_discount: bool = excludeDiscount
        self._exclude_markup: bool = excludeMarkup

        if measureUnit is not None: 
            self._measure_unit: MeasureUnit = MeasureUnit(**measureUnit)
        else:
            self._measure_unit = None

        self._minimal_price: float = minimalPrice
        self._name: str = name
        self._pack: float = pack
        self._price: float = price
        self._ratio: float = ratio
        self._recipe: str = recipe
        self._title: str = itemTitle
        self._selling_type: SellingType = convert_str_to_selling_type(sellingType)
        self._store_quantity_kg: float = storeQuantityKg

        if storeItemTag is not None:
            self._store_item_tag = StoreItemTag(**storeItemTag)
        else:
            self._store_item_tag = None

        self._parent_id = parentId

        if parentItem is not None: 
            self._parent_item = DishCategory(**parentItem)
        else:
            self._parent_item = None



from quick_resto_API.quick_resto_objects.modules.core.payment import Payment
from quick_resto_API.quick_resto_objects.modules.front.table_scheme import TableScheme
from quick_resto_API.quick_resto_objects.modules.front.terminal import Terminal
from quick_resto_API.quick_resto_objects.modules.front.shift import Shift
from quick_resto_API.quick_resto_objects.modules.personnel.employee import Employee
from quick_resto_API.quick_resto_objects.quick_resto_object import QuickRestoObject
from quick_resto_API.quick_resto_objects.modules.warehouse.sale_place import SalePlace


class OrderInfo(QuickRestoObject):
    @property
    def comment(self) -> str:
        return self._comment

    @property
    def create_date(self) -> str:
        return self._create_date

    @property
    def document_number(self) -> int:
        return self._document_number

    @property
    def discount_percent(self) -> int:
        return self._discount_percent

    @property
    def cashier(self) -> Employee:
        return self._cashier

    @property
    def ext_id(self) -> str:
        return self._ext_id

    @property
    def front(self) -> Terminal:
        return self._front

    @property
    def front_sum(self) -> float:
        return self._front_sum

    @property
    def front_total_absolute_charge(self) -> float:
        return self._front_total_absolute_charge

    @property
    def front_total_absolute_discount(self) -> float:
        return self._front_total_absolute_discount

    @property
    def front_total_bonuses(self) -> float:
        return self._front_total_bonuses

    @property
    def front_total_card(self) -> float:
        return self._front_total_card

    @property
    def front_total_cash_minus_discount(self) -> float:
        return self._front_total_cash_minus_discount

    @property
    def front_total_discount(self) -> float:
        return self._front_total_discount

    @property
    def front_total_price(self) -> float:
        return self._front_total_price

    @property
    def kkm_terminal_name(self) -> str:
        return self._kkm_terminal_name

    @property
    def original_order_doc_id(self) -> str:
        return self._original_order_doc_id

    @property
    def create_terminal_sale_place(self) -> SalePlace:
        return self._create_terminal_sale_place

    @property
    def precheck_doc_id(self) -> str:
        return self._precheck_doc_id

    @property
    def reopened_order_doc_id(self) -> str:
        return self._reopened_order_doc_id

    @property
    def returned(self) -> bool:
        return self._returned

    @property
    def return_order_doc_id(self) -> str:
        return self._return_order_doc_id

    @property
    def shift(self) -> Shift:
        return self._shift

    @property
    def shift_id(self) -> str:
        return self._shift_id

    @property
    def table_scheme(self) -> TableScheme:
        return self._table_scheme

    @property
    def table_order_doc_id(self) -> str:
        return self._table_order_doc_id

    @property
    def total_sum(self) -> float:
        return self._total_sum

    @property
    def waiter(self) -> Employee:
        return self._waiter

    @property
    def payments(self) -> list:
        return self._payments

    def __init__(self, comment: str = None, createDate: str = None, documentNumber: int = None,
                 discountPercent: float = None, cashier: dict = None, extId: str = None,
                 front: dict = None, frontSum: float = None, frontTotalAbsoluteCharge: float = None,
                 frontTotalAbsoluteDiscount: float = None, frontTotalBonuses: float = None,
                 frontTotalCard: float = None, frontTotalCashMinusDiscount: float = None, frontTotalDiscount: float = None,
                 frontTotalPrice: float = None, createTerminalSalePlace:dict = None,
                 kkmTerminalName: str = None, originalOrderDocId: str = None, precheckDocId: str = None,
                 reopenedOrderDocId: str = None, returned: bool = None,
                 returnOrderDocId: str = None, shift: dict = None, shiftId: str = None, tableScheme: dict = None,
                 tableOrderDocId: str = None, totalSum: float = None,
                 waiter: dict = None,payments:list = None, **kwargs):
        class_name = "ru.edgex.quickresto.modules.front.orders.OrderInfo"

        super().__init__(class_name=class_name, **kwargs)

        self._comment: str = comment
        self._create_date: str = createDate
        self._document_number: int = documentNumber
        self._discount_percent: float = discountPercent
        
        if cashier is not None:
            self._cashier = Employee(**cashier)
        else:
            self._cashier = None
        
        self._ext_id: str = extId

        if front is not None: 
            self._front = Terminal(**front)
        else:
            self._front = None

        if createTerminalSalePlace is not None:
            self._create_terminal_sale_place = SalePlace(**createTerminalSalePlace)
        else:
            self._create_terminal_sale_place = None

        self._front_sum: float = frontSum
        self._front_total_absolute_charge: float = frontTotalAbsoluteCharge
        self._front_total_absolute_discount: float = frontTotalAbsoluteDiscount
        self._front_total_bonuses: float = frontTotalBonuses
        self._front_total_card: float = frontTotalCard
        self._front_total_cash_minus_discount: float = frontTotalCashMinusDiscount
        self._front_total_discount: float = frontTotalDiscount
        self._front_total_price: float = frontTotalPrice
        self._kkm_terminal_name: str = kkmTerminalName
        self._original_order_doc_id: str = originalOrderDocId
        self._precheck_doc_id: str = precheckDocId
        self._reopened_order_doc_id: str = reopenedOrderDocId
        self._returned: bool = returned
        self._return_order_doc_id: str = returnOrderDocId

        if shift is not None: 
            self._shift = Shift(**shift)
        else:
            self._shift = None

        self._shift_id: str = shiftId
        
        if tableScheme is not None: 
            self._table_scheme = TableScheme(**tableScheme)
        else:
            self._table_scheme = None

        self._table_order_doc_id: str = tableOrderDocId
        self._total_sum: float = totalSum

        if waiter is not None: 
            self._waiter = Employee(**waiter)
        else:
            self._waiter = None

        if payments is not None:
            self._payments = [Payment(**payment) for payment in payments]
        else:
            self._payments = None



from quick_resto_API.quick_resto_objects.modules.crm.account_type import AccountType
from quick_resto_API.quick_resto_objects.modules.crm.account_balance import AccountBalance
from quick_resto_API.quick_resto_objects.quick_resto_object import QuickRestoObject


class CustomerAccount(QuickRestoObject):
    def __init__(self, accountBalance: dict = None, accountState: bool = None, accountType: dict = None,
                 convertBonuses: bool = None, deleted: bool = None,
                 hidden: bool = None, **kwargs):
        class_name = 'ru.edgex.quickresto.modules.crm.accounting.account.CustomerAccount'

        super().__init__(id=0, class_name=class_name, **kwargs)

        if accountBalance is not None: 
            self._accounts = AccountBalance(**accountBalance)
        else:
            self._accounts = None

        self._account_state: bool = accountState

        if accountType is not None: 
            self._account_type: AccountType = AccountType(**accountType)
        else:
            self._account_type = None

        self._convert_bonuses: bool = convertBonuses
        self._deleted: bool = deleted
        self._hidden: bool = hidden



from quick_resto_API.quick_resto_objects.quick_resto_object import QuickRestoObject

class User(QuickRestoObject):
    @property
    def user_kind(self) -> str:
        return self._user_kind

    @property
    def last_name(self) -> str:
        return self._last_name

    @property
    def login(self) -> str:
        return self._login

    @property
    def language(self) -> dict:
        return self._language

    @property
    def deleted(self) -> bool:
        return self._deleted

    @property
    def hidden(self) -> bool:
        return self._hidden

    @property
    def last_login_time(self) -> str:
        return self._last_login_time

    @property
    def tokens(self) -> list:
        return self._tokens

    def __init__(self, userKind: str = None, lastName: str = None, login: str = None, language: dict = None, deleted: bool = None, 
                hidden: bool = None, lastLoginTime: str = None, tokens: list = None, **kwargs):
        class_name = ""
        super().__init__(class_name=class_name, **kwargs)

        self._user_kind: str = userKind
        self._last_name: str = lastName
        self._login: str = login
        self._language: dict = language
        self._deleted: bool = deleted
        self._hidden: bool = hidden
        self._last_login_time: str = lastLoginTime
        self._tokens: list = tokens


import json
from enum import Enum

import quick_resto_API.quick_resto_objects.styler as styler

class QuickRestoObject(object):
    @property
    def id(self) -> int:
        return self._id

    @property
    def global_id(self) -> int:
        return self._global_id

    @property
    def class_name(self) -> str:
        return self._class_name

    def get_json_object(self) -> dict:
        as_dict = self.__dict__
        result = dict()

        for key, value in as_dict.items():
            parameter_name = styler.to_camel_case(key)

            if issubclass(type(value), QuickRestoObject):
                result[parameter_name] = value.get_json_object()
            elif issubclass(type(value), list):
                list_of_subobjects = []

                for obj in value:
                    if issubclass(type(obj), QuickRestoObject):
                        list_of_subobjects.append(obj.get_json_object())

                result[parameter_name] = list_of_subobjects
            else:
                result[parameter_name] = value

        return result

    def __str__(self) -> str:
        return self.__repr__()

    def __repr__(self) -> str:
        return json.dumps(self.get_json_object(), cls=QuickRestoObjectEncoder, indent=4, ensure_ascii=False)

    def __init__(self, class_name: str, id: int = 0,  className: str = "", _id: int = 0, **kwargs):
        self._id: int = id
        self._global_id: int = _id

        # Server do not always return className
        if className == "":
            self._class_name: str = class_name
        else:
            self._class_name: str = className


class QuickRestoObjectEncoder(json.JSONEncoder):
    def default(self, obj):

        if isinstance(obj, QuickRestoObject):
            return obj.__dict__
        elif isinstance(obj, Enum):
            return obj.value

        return json.JSONEncoder.default(self, obj)



from quick_resto_API.quick_resto_objects.modules.warehouse.businessman import Businessman
from quick_resto_API.quick_resto_objects.modules.warehouse.natural_person import NaturalPerson
from quick_resto_API.quick_resto_objects.modules.warehouse.organization import Organization
from quick_resto_API.quick_resto_objects.modules.warehouse.provider_group import ProviderGroup
from quick_resto_API.quick_resto_objects.modules.warehouse.store import Store
from quick_resto_API.quick_resto_objects.quick_resto_object import QuickRestoObject


class IncomingInvoice(QuickRestoObject):
    @property
    def document_number(self) -> str:
        return self._document_number

    @property
    def invoice_date(self) -> str:
        return self._invoice_date

    @property
    def provider(self) -> QuickRestoObject:
        return self._provider

    @property
    def store(self) -> Store:
        return self._store

    @property
    def processed(self) -> bool:
        return self._processed

    @property
    def total_sum(self) -> float:
        return self._total_sum

    @property
    def total_sum_wo_nds(self) -> float:
        return self._total_sum_wo_nds

    @property
    def total_nds(self) -> float:
        return self._total_nds

    @property
    def paid(self) -> bool:
        return self._paid

    @property
    def total_amount(self) -> float:
        return self._total_amount

    def __init__(self, documentNumber: str, invoiceDate: str, processed: bool, totalSum: float, totalSumWoNds: float,
                 totalNds: float, paid: bool, totalAmount: float, provider: dict = None, store: dict = None, **kwargs):
        class_name = "ru.edgex.quickresto.modules.warehouse.documents.incoming.IncomingInvoice"

        super().__init__(class_name=class_name, **kwargs)

        self._paid = paid
        self._document_number = documentNumber
        self._invoice_date = invoiceDate
        self._processed = processed
        self._total_sum = totalSum
        self._total_sum_wo_nds = totalSumWoNds
        self._total_nds = totalNds
        self._total_amount = totalAmount

        if store is not None:
            self._store = Store(**store)

        if provider is not None:
            if provider is Businessman:
                self._provider = Businessman(**provider)
            elif provider is NaturalPerson:
                self._provider = NaturalPerson(**provider)
            elif provider is Organization:
                self._provider = Organization
            elif provider is ProviderGroup:
                self._provider = ProviderGroup(**provider)
            else:
                provider = None
        else:
                provider = None



from quick_resto_API.quick_resto_objects.modules.front.device_command import DeviceCommand
from quick_resto_API.quick_resto_objects.quick_resto_object import QuickRestoObject


class VirtualKkmTerminal(QuickRestoObject):
    @property
    def device_id(self) -> str:
        return self._device_id

    @property
    def manufacturer(self) -> str:
        return self._manufacturer

    @property
    def model(self) -> str:
        return self._model

    @property
    def connected(self) -> bool:
        return self._connected

    @property
    def name(self) -> str:
        return self._name

    @property
    def deleted(self) -> bool:
        return self._deleted

    @property
    def activate_on_current_terminal(self) -> bool:
        return self._activate_on_current_terminal

    @property
    def state(self) -> str:
        return self._state

    @property
    def command(self) -> DeviceCommand:
        return self._command

    def __init__(self, deviceId: str = None, manufacturer: str = None, model: str = None, connected: bool = None,
                 name: str = None, deleted: bool = None,
                 activateOnCurrentTerminal: bool = None, state: str = None, command: dict = None, **kwargs):
        class_name = "ru.edgex.quickresto.modules.front.terminals.kkm.VirtualKkmTerminal"

        super().__init__(class_name=class_name, **kwargs)

        self._device_id: str = deviceId
        self._manufacturer: str = manufacturer
        self._model: str = model
        self._connected: bool = connected
        self._name: str = name
        self._deleted: bool = deleted
        self._activate_on_current_terminal: bool = activateOnCurrentTerminal
        self._state: str = state

        if command is not None: 
            self._command = DeviceCommand(**command)
        else:
            self._command = None



from enum import Enum

from quick_resto_API.quick_resto_objects.modules.front.table_scheme import TableScheme
# from quick_resto_API.quick_resto_objects.modules.warehouse.cooking_place import CookingPlace
# from quick_resto_API.quick_resto_objects.modules.warehouse.sale_place import SalePlace
from quick_resto_API.quick_resto_objects.quick_resto_object import QuickRestoObject


class TerminalSubtype(Enum):
    REGISTER = "REGISTER"
    WAITER = "WAITER"
    NONE = "NONE"


class TerminalType(Enum):
    QUICK_POS = "QUICK_POS"
    QUICK_RESTO = "QUICK_RESTO"
    KITCHEN = "KITCHEN"
    NONE = "NONE"


StrToTerminalSubtype = {
    TerminalSubtype.REGISTER.value: TerminalSubtype.REGISTER,
    TerminalSubtype.WAITER.value: TerminalSubtype.WAITER,
}

StrToTerminalType = {
    TerminalType.QUICK_POS.value: TerminalType.QUICK_POS,
    TerminalType.QUICK_RESTO.value: TerminalType.QUICK_RESTO,
    TerminalType.KITCHEN.value: TerminalType.KITCHEN,
}


def convert_str_to_terminal_subtype(value: str) -> TerminalSubtype:
    if value in StrToTerminalSubtype.keys():
        return StrToTerminalSubtype[value]

    return TerminalSubtype.NONE


def convert_str_to_terminal_type(value: str) -> TerminalType:
    if value in StrToTerminalType.keys():
        return StrToTerminalType[value]

    return TerminalType.NONE


class Terminal(QuickRestoObject):
    @property
    def cooking_place(self) -> dict:
        return self._cooking_place

    @property
    def deleted(self) -> bool:
        return self._deleted

    @property
    def device_manufacturer(self) -> str:
        return self._device_manufacturer

    @property
    def device_model(self) -> str:
        return self._device_model

    @property
    def last_sync(self) -> str:
        return self._last_sync

    @property
    def name(self) -> str:
        return self._name

    @property
    def online(self) -> bool:
        return self._online

    @property
    def registration_date(self) -> str:
        return self._registration_date

    @property
    def sale_place(self) -> dict:
        return self._sale_place

    @property
    def short_name(self) -> str:
        return self._short_name

    @property
    def table_scheme(self) -> dict:
        return self._table_scheme

    @property
    def terminal_subtype(self) -> TerminalSubtype:
        return self._terminal_subtype

    @property
    def terminal_type(self) -> TerminalType:
        return self._terminal_type

    def __init__(self, cookingPlace: dict = None, deleted: bool = None, deviceManufacturer: str = None,
                 deviceModel: str = None,
                 lastSync: str = None, name: str = None, online: bool = None, registrationDate: str = None,
                 salePlace: dict = None,
                 shortName: str = None, tableScheme: dict = None, terminalSubtype: dict = None,
                 terminalType: dict = None, **kwargs):
        class_name = "ru.edgex.quickresto.modules.front.terminals.ipad.Terminal"

        super().__init__(class_name=class_name, **kwargs)

        self._cooking_place = cookingPlace

        self._deleted: bool = deleted
        self._device_manufacturer: str = deviceManufacturer
        self._device_model: str = deviceModel
        self._last_sync: str = lastSync
        self._name: str = name
        self._online: bool = online
        self._registration_date: str = registrationDate

        self._sale_place = salePlace

        self._short_name: str = shortName

        if tableScheme is not None: 
            self._table_scheme = TableScheme(**tableScheme)
        else:
            self._table_scheme = None

        self._terminal_subtype = convert_str_to_terminal_subtype(terminalSubtype)
        self._terminal_type = convert_str_to_terminal_type(terminalType)



from quick_resto_API.quick_resto_objects.modules.core.measure_unit import MeasureUnit
from quick_resto_API.quick_resto_objects.modules.core.store_item_tag import StoreItemTag
from quick_resto_API.quick_resto_objects.quick_resto_object import QuickRestoObject


class SingleCategory(QuickRestoObject):
    @property
    def name(self) -> str:
        return self._name

    @property
    def measure_unit(self) -> MeasureUnit:
        return self._measure_unit

    @property
    def store_item_tag(self) -> StoreItemTag:
        return self._store_item_tag

    @property
    def color(self) -> str:
        return self._color

    @property
    def display_on_terminal(self) -> bool:
        return self._display_on_terminal

    @property
    def item_title(self) -> str:
        return self._item_title

    def __init__(self, version: int = None, serverRegisterTime: str = None, name: str = None, measureUnit: dict = None,
                 storeItemTag: dict = None,
                 color: str = None, displayOnTerminal: bool = None, itemTitle: str = None, **kwargs):
        class_name = "ru.edgex.quickresto.modules.warehouse.nomenclature.singleproduct.SingleCategory"
        super().__init__(class_name=class_name, **kwargs)

        self._version: int = version
        self._server_register_time: str = serverRegisterTime
        self._name: str = name

        if measureUnit is not None: 
            self._measure_unit: MeasureUnit = MeasureUnit(**measureUnit)
        else:
            self._measure_unit = None

        if storeItemTag is not None:
            self._store_item_tag = StoreItemTag(**storeItemTag)
        else:
            self._store_item_tag = None

        self._color: str = color
        self._display_on_terminal: bool = displayOnTerminal
        self._item_title: str = itemTitle



from quick_resto_API.quick_resto_objects.platform.right import Right
from quick_resto_API.quick_resto_objects.quick_resto_object import QuickRestoObject


class RightLink(QuickRestoObject):
    @property
    def right(self) -> Right:
        return self._right

    def __init__(self, right: dict = None, **kwargs):
        class_name = "ru.edgex.platform.service.user.RightLink"

        super().__init__(class_name=class_name, **kwargs)

        if right is not None: 
            self._right = Right(**right)
        else:
            self._right = None



from quick_resto_API.quick_resto_objects.modules.front.terminal import Terminal
from quick_resto_API.quick_resto_objects.modules.warehouse.store import Store
from quick_resto_API.quick_resto_objects.quick_resto_object import QuickRestoObject


class CookingPlace(QuickRestoObject):
    @property
    def send_signal(self) -> bool:
        return self._send_signal

    @property
    def store(self) -> Store:
        return self._store

    @property
    def title(self) -> str:
        return self._title

    @property
    def backup_target_device(self) -> Terminal:
        return self._backup_target_device

    @property
    def doublicating_target_Device(self) -> Terminal:
        return self._doublicating_target_Device

    @property
    def target_Device(self) -> Terminal:
        return self._target_Device

    def __init__(self, sendSignal: bool = None, title: str = None, store: dict = None, backupTargetDevice: dict = None,
                 doublicatingTargetDevice: dict = None, targetDevice: dict = None, **kwargs):
        class_name: str = "ru.edgex.quickresto.modules.warehouse.nomenclature.cooking_place.CookingPlace"

        super().__init__(class_name=class_name, **kwargs)
        self._send_signal = sendSignal
        self._title = title

        if backupTargetDevice is not None: 
            self._backup_target_device = Terminal(**backupTargetDevice)
        else:
            self._backup_target_device = None

        if targetDevice is not None: 
            self._target_Device = Terminal(**targetDevice)
        else:
            self._target_Device = None

        if store is not None:
            self._store = Store(**store)
        else:
            self._store = None

        if doublicatingTargetDevice is not None:
            self._doublicating_target_Device = Terminal(**doublicatingTargetDevice)
        else:
            self._doublicating_target_Device = None



from quick_resto_API.quick_resto_objects.modules.core.measure_unit import MeasureUnit
from quick_resto_API.quick_resto_objects.modules.core.store_item_tag import StoreItemTag
from quick_resto_API.quick_resto_objects.modules.warehouse.dish_sale import DishSale
from quick_resto_API.quick_resto_objects.quick_resto_object import QuickRestoObject


class ModifierGroup(QuickRestoObject):
    @property
    def name(self) -> str:
        return self._name

    @property
    def deleted(self) -> bool:
        return self._deleted

    @property
    def measure_unit(self) -> MeasureUnit:
        return self._measure_unit

    @property
    def color(self) -> str:
        return self._color

    @property
    def display_on_terminal(self) -> bool:
        return self._display_on_terminal

    @property
    def min_value(self) -> int:
        return self._min_value

    @property
    def max_value(self) -> int:
        return self._max_value

    @property
    def with_dish(self) -> bool:
        return self._with_dish

    @property
    def modifier_sales(self) -> list:
        return self._modifier_sales

    @property
    def item_title(self) -> str:
        return self._item_title

    @property
    def store_item_tag(self) -> StoreItemTag:
        return self._store_item_tag

    def __init__(self, name: str = None, measureUnit: dict = None, color: str = None, displayOnTerminal: bool = None,
                 minValue: int = None, maxValue: int = None, withDish: bool = None,
                 modifierSales: list = None, itemTitle: str = None, deleted: bool = None, storeItemTag=None, **kwargs):
        class_name = "ru.edgex.quickresto.modules.warehouse.nomenclature.mods.ModifierGroup"

        super().__init__(class_name=class_name, **kwargs)

        self._name = name
        self._deleted = deleted

        if measureUnit is not None: 
            self._measure_unit: MeasureUnit = MeasureUnit(**measureUnit)
        else:
            self._measure_unit = None

        self._color = color
        self._display_on_terminal = displayOnTerminal
        self._min_value = minValue
        self._max_value = maxValue
        self._with_dish = withDish

        if modifierSales is not None:
            self._modifier_sales: list = [DishSale(**dish_sale) for dish_sale in modifierSales]
        else:
            self._modifier_sales = None
        
        self._item_title = itemTitle

        if storeItemTag is not None:
            self._store_item_tag = StoreItemTag(**storeItemTag)
        else:
            self._store_item_tag = None



from quick_resto_API.quick_resto_objects.modules.core.measure_unit import MeasureUnit
from quick_resto_API.quick_resto_objects.modules.core.store_item_tag import StoreItemTag
from quick_resto_API.quick_resto_objects.quick_resto_object import QuickRestoObject


class SemiProduct(QuickRestoObject):
    @property
    def name(self) -> str:
        return self._name

    @property
    def article(self) -> str:
        return self._article

    @property
    def measure_unit(self) -> MeasureUnit:
        return self._measure_unit

    @property
    def store_item_tag(self) -> StoreItemTag:
        return self._store_item_tag

    @property
    def ratio(self) -> float:
        return self._ratio

    @property
    def minimal_price(self) -> float:
        return self._minimal_price

    @property
    def exclude_discount(self) -> bool:
        return self._exclude_discount

    @property
    def exclude_markup(self) -> bool:
        return self._exclude_markup

    @property
    def store_quantity_kg(self) -> float:
        return self._store_quantity_kg

    @property
    def limit(self) -> float:
        return self._limit

    @property
    def item_title(self) -> str:
        return self._item_title

    @property
    def base_price_in_list(self) -> float:
        return self._base_price_in_list

    @property
    def pack(self) -> float:
        return self._pack

    @property
    def recipe(self) -> str:
        return self._recipe

    @property
    def parent_id(self) -> int:
        return self._parent_id

    @property
    def parent_item(self) -> dict:
        return self._parent_item

    def __init__(self, version: int = None, serverRegisterTime: str = None, name: str = None, article: str = None,
                 measureUnit: dict = None, storeItemTag: dict = None, parentId:int = None, parentItem:dict = None,
                 ratio: float = None, minimalPrice: float = None, excludeDiscount: bool = None,
                 excludeMarkup: bool = None, storeQuantityKg: float = None,
                 limit: float = None, itemTitle: str = None, basePriceInList: float = None, pack: float = None,
                 recipe: str = None, **kwargs):
        class_name = "ru.edgex.quickresto.modules.warehouse.nomenclature.semiproduct.SemiProduct"
        super().__init__(class_name=class_name, **kwargs)

        self._version: int = version
        self._server_register_time: str = serverRegisterTime
        self._name: str = name
        self._article: str = article
        
        if measureUnit is not None: 
            self._measure_unit: MeasureUnit = MeasureUnit(**measureUnit)
        else:
            self._measure_unit = None

        if storeItemTag is not None:
            self._store_item_tag = StoreItemTag(**storeItemTag)
        else:
            self._store_item_tag = None

        self._ratio: float = ratio
        self._minimal_price: float = minimalPrice
        self._exclude_discount: bool = excludeDiscount
        self._exclude_markup: bool = excludeMarkup
        self._store_quantity_kg: float = storeQuantityKg
        self._limit: float = limit
        self._item_title: str = itemTitle
        self._base_price_in_list: float = basePriceInList
        self._pack: float = pack
        self._recipe: str = recipe
        
        self._parent_id = parentId
        self._parent_item = parentItem


from quick_resto_API.quick_resto_objects.modules.core.payment_types import PaymentType
from quick_resto_API.quick_resto_objects.quick_resto_object import QuickRestoObject

class Payment(QuickRestoObject):
    @property
    def payment_type(self) -> PaymentType:
        return self._payment_type

    @property
    def partial_allowed(self) -> bool:
        return self._partial_allowed

    @property
    def require_admin_confirmation(self) -> bool:
        return self._require_admin_confirmation

    @property
    def require_customer_confirmation(self) -> bool:
        return self._require_customer_confirmation

    @property
    def customer_type(self) -> str:
        return self._customer_type

    @property
    def amount(self) -> float:
        return self._amount

    def __init__(self,paymentType:dict, partialAllowed: bool = None, requireAdminConfirmation: bool = None,
                 requireCustomerConfirmation: bool = None, customerType: str = None, amount:float = None, **kwargs):
        class_name = ""

        super().__init__(class_name=class_name, **kwargs)

        if paymentType is not None:
            self._payment_type = PaymentType(**paymentType)
        else:
            self._payment_type = None

        self._partial_allowed: bool = partialAllowed
        self._require_admin_confirmation: bool = requireAdminConfirmation
        self._require_customer_confirmation: bool = requireCustomerConfirmation
        self._customer_type: str = customerType
        self._amount = amount



from quick_resto_API.quick_resto_objects.modules.front.table import Table
from quick_resto_API.quick_resto_objects.quick_resto_object import QuickRestoObject


class Hall(QuickRestoObject):
    @property
    def deleted(self) -> bool:
        return self._deleted

    @property
    def tables(self) -> list:
        return self._tables

    @property
    def width(self) -> int:
        return self._width

    @property
    def height(self) -> int:
        return self._height

    @property
    def title(self) -> str:
        return self._title

    def __init__(self, deleted: bool = None, tables: list = None, width: int = None,
                 height: int = None, title: str = None, **kwargs):
        class_name = "ru.edgex.quickresto.modules.front.tablemanagement.hall.Hall"
        
        super().__init__(class_name=class_name, **kwargs)

        self._deleted: bool = deleted
        
        if tables is not None: 
            self._tables: list = [Table(**value) for value in tables]
        else:
            self._tables = None

        self._width: int = width
        self._height: int = height
        self._title: str = title



from quick_resto_API.quick_resto_objects.modules.warehouse.sale_place import SalePlace
from quick_resto_API.quick_resto_objects.platform.role import Role
from quick_resto_API.quick_resto_objects.quick_resto_object import QuickRestoObject


class PaymentType(QuickRestoObject):
    @property
    def name(self) -> str:
        return self._name

    @property
    def type(self) -> str:
        return self._type

    @property
    def operation_type(self) -> str:
        return self._operation_type

    @property
    def partial_allowed(self) -> bool:
        return self._partial_allowed

    @property
    def seq(self) -> int:
        return self._seq

    @property
    def require_admin_confirmation(self) -> bool:
        return self._require_admin_confirmation

    @property
    def require_customer_confirmation(self) -> bool:
        return self._require_customer_confirmation

    @property
    def customer_type(self) -> str:
        return self._customer_type

    @property
    def payment_mechanism_web(self) -> str:
        return self._payment_mechanism_web

    @property
    def allowed_organizations_web(self) -> list:
        return self._allowed_organizations_web

    @property
    def allowed_sale_places_web(self) -> list:
        return self._allowed_sale_places_web

    @property
    def employee_roles_web(self) -> list:
        return self._employee_roles_web

    @property
    def concrete_employees_web(self) -> list:
        return self._concrete_employees_web

    @property
    def allowed_user_roles_web(self) -> list:
        return self._allowed_user_roles_web

    @property
    def allowed_concrete_users_web(self) -> list:
        return self._allowed_concrete_users_web

    @property
    def guest_groups_web(self) -> list:
        return self._guest_groups_web

    @property
    def concrete_guests_web(self) -> list:
        return self._concrete_guests_web

    @property
    def partner_groups_web(self) -> list:
        return self._partner_groups_web

    @property
    def concrete_partners_web(self) -> list:
        return self._concrete_partners_web

    def __init__(self, name: str = None, type: str = None, operationType: str = None, partialAllowed: bool = None,
                 seq: int = None, requireAdminConfirmation: bool = None,
                 requireCustomerConfirmation: bool = None, customerType: str = None, paymentMechanismWeb: str = None,
                 allowedOrganizationsWeb: list = None,
                 allowedSalePlacesWeb: list = None, employeeRolesWeb: list = None, concreteEmployeesWeb: list = None,
                 allowedUserRolesWeb: list = None,
                 allowedConcreteUsersWeb: list = None, guestGroupsWeb: list = None, concreteGuestsWeb: list = None,
                 partnerGroupsWeb: list = None,
                 concretePartnersWeb: list = None, **kwargs):
        class_name = "ru.edgex.quickresto.modules.core.dictionaries.paymenttypes.PaymentType"

        super().__init__(class_name=class_name, **kwargs)

        self._name: str = name
        self._type: str = type
        self._operation_type: str = operationType
        self._partial_allowed: bool = partialAllowed
        self._seq: int = seq
        self._require_admin_confirmation: bool = requireAdminConfirmation
        self._require_customer_confirmation: bool = requireCustomerConfirmation
        self._customer_type: str = customerType
        self._payment_mechanism_web: str = paymentMechanismWeb
        self._allowed_organizations_web: list = allowedOrganizationsWeb

        if allowedSalePlacesWeb is not None: 
            self._allowed_sale_places_web: list = [SalePlace(**place) for place in allowedSalePlacesWeb]
        else:
            self._allowed_sale_places_web = None
        
        self._employee_roles_web: list = employeeRolesWeb
        self._concrete_employees_web: list = concreteEmployeesWeb

        if allowedUserRolesWeb is not None: 
            self._allowed_user_roles_web: list = [Role(**value) for value in allowedUserRolesWeb]
        else:
            self._allowed_user_roles_web = None

        self._allowed_concrete_users_web: list = allowedConcreteUsersWeb
        self._guest_groups_web: list = guestGroupsWeb
        self._concrete_guests_web: list = concreteGuestsWeb
        self._partner_groups_web: list = partnerGroupsWeb
        self._concrete_partners_web: list = concretePartnersWeb



from quick_resto_API.quick_resto_objects.modules.warehouse.store import Store
from quick_resto_API.quick_resto_objects.quick_resto_object import QuickRestoObject


class CookingInvoice(QuickRestoObject):
    @property
    def from_store(self) -> Store:
        return self._from_store

    @property
    def document_number(self) -> str:
        return self._document_number

    @property
    def invoice_date(self) -> str:
        return self._invoice_date

    @property
    def store(self) -> Store:
        return self._store

    @property
    def processed(self) -> bool:
        return self._processed

    @property
    def total_sum(self) -> float:
        return self._total_sum

    @property
    def total_sum_wo_nds(self) -> float:
        return self._total_sum_wo_nds

    @property
    def total_nds(self) -> float:
        return self._total_nds

    @property
    def comment(self) -> str:
        return self._comment

    @property
    def total_amount(self) -> float:
        return self._total_amount

    def __init__(self, fromStore: dict = None, documentNumber: str = None, invoiceDate: str = None, store: dict = None,
                 processed: bool = None, totalSum: float = None,
                 totalSumWoNds: float = None, totalNds: float = None, comment: str = None, totalAmount: float = None,
                 **kwargs):
        class_name = "ru.edgex.quickresto.modules.warehouse.documents.cooking.CookingInvoice"

        super().__init__(class_name=class_name, **kwargs)

        if fromStore is not None: 
            self._from_store = Store(**fromStore)
        else:
            self._from_store = None

        self._document_number: str = documentNumber
        self._invoice_date: str = invoiceDate

        if store is not None: 
            self._store = Store(**store)
        else:
            self._store = None

        self._processed: bool = processed
        self._total_sum: float = totalSum
        self._total_sum_wo_nds: float = totalSumWoNds
        self._total_nds: float = totalNds
        self._comment: str = comment
        self._total_amount: float = totalAmount



from enum import Enum

from quick_resto_API.quick_resto_objects.quick_resto_object import QuickRestoObject


class SystemOrderDiscardReason(Enum):
    CancellationOfWriteOff = "CancellationOfWriteOff",
    CancellationWithoutWriteOffs = "CancellationWithoutWriteOffs",
    NONE = "NONE"


StrToSystemOrderDiscardReason = {
    SystemOrderDiscardReason.CancellationOfWriteOff.value: SystemOrderDiscardReason.CancellationOfWriteOff,
    SystemOrderDiscardReason.CancellationWithoutWriteOffs.value: SystemOrderDiscardReason.CancellationWithoutWriteOffs,
    SystemOrderDiscardReason.NONE.value: SystemOrderDiscardReason.NONE
}


def convert_str_to_system_order_discard_reason(value: str) -> SystemOrderDiscardReason:
    if value in StrToSystemOrderDiscardReason.keys():
        return StrToSystemOrderDiscardReason[value]

    return SystemOrderDiscardReason.NONE


class OrderDiscardReason(QuickRestoObject):
    @property
    def deleted(self) -> bool:
        return self._deleted

    @property
    def save_to_terminals(self) -> bool:
        return self._save_to_terminals

    @property
    def description(self) -> str:
        return self._description

    @property
    def title(self) -> str:
        return self._title

    @property
    def use_comment(self) -> bool:
        return self._use_comment

    @property
    def withdraw_from_store(self) -> bool:
        return self._withdraw_from_store

    @property
    def system_order_discard_reason(self) -> SystemOrderDiscardReason:
        return self._system_order_discard_reason

    def __init__(self, saveToTerminals: bool = None, description: str = None, systemOrderDiscardReason: str = None, title: str = None,
                 useComment: bool = None,
                 withdrawFromStore: bool = None, deleted: bool = None, **kwargs):
        class_name: str = "ru.edgex.quickresto.modules.core.dictionaries.orderdiscardreasons.OrderDiscardReason"

        super().__init__(class_name=class_name, **kwargs)

        self._deleted = deleted
        self._save_to_terminals = saveToTerminals
        self._description = description
        self._title = title
        self._use_comment = useComment
        self._withdraw_from_store = withdrawFromStore
        self._system_order_discard_reason = convert_str_to_system_order_discard_reason(systemOrderDiscardReason)



from enum import Enum

from quick_resto_API.quick_resto_objects.modules.warehouse.dish import Dish
from quick_resto_API.quick_resto_objects.modules.warehouse.dish_category import DishCategory
from quick_resto_API.quick_resto_objects.modules.warehouse.modifier_group import ModifierGroup
from quick_resto_API.quick_resto_objects.modules.warehouse.sale_place import SalePlace
from quick_resto_API.quick_resto_objects.quick_resto_object import QuickRestoObject


class TypeDiscount(Enum):
    ABSOLUTE = "ABSOLUTE"
    PERCENT = "PERCENT"
    NONE = "NONE"


StrToTypeDiscount = {
    TypeDiscount.ABSOLUTE.value: TypeDiscount.ABSOLUTE,
    TypeDiscount.PERCENT.value: TypeDiscount.PERCENT,
}

def convert_str_to_type_discount(value: str) -> TypeDiscount:
    if value in StrToTypeDiscount.keys():
        return TypeDiscount[value]

    return TypeDiscount.NONE


class FixedDiscount(QuickRestoObject):
    @property
    def name(self) -> str:
        return self._name

    @property
    def deleted(self) -> bool:
        return self._deleted

    @property
    def type_discount(self) -> TypeDiscount:
        return self._type_discount

    @property
    def value(self) -> float:
        return self._value

    @property
    def categories(self) -> set:
        return self._categories

    @property
    def dishes(self) -> set:
        return self._dishes

    @property
    def tags(self) -> set:
        return self._tags

    @property
    def modifier_groups(self) -> list:
        return self._modifier_groups

    @property
    def sale_places(self) -> set:
        return self._sale_places

    @property
    def index(self) -> int:
        return self._index

    def __init__(self, name: str = None, deleted: bool = None, typeDiscount: str = None, value: float = None,
                 categories: set = None, dishes: set = None,
                 tags: set = None, modifierGroups: list = None, salePlaces: set = None, index: int = None, **kwargs):
        class_name = "ru.edgex.quickresto.modules.crm.settings.fixed.FixedDiscount"

        super().__init__(class_name=class_name, **kwargs)

        self._name: str = name
        self._deleted: bool = deleted
        self._type_discount = convert_str_to_type_discount(typeDiscount)
        self._value: float = value

        if categories is not None: 
            self._categories: set = [DishCategory(**category) for category in categories]
        else:
            self._categories = None

        if dishes is not None: 
            self._dishes: set = [Dish(**dish) for dish in dishes]
        else:
            self._dishes

        if tags is not None: 
            self._tags: set = tags
        else:
            self._tags = None

        if modifierGroups is not None: 
            self._modifier_groups: list = [ModifierGroup(**group) for group in modifierGroups]
        else:
            self._modifier_groups = None

        if salePlaces is not None: 
            self._sale_places: set = [SalePlace(**place) for place in salePlaces]
        else:
            self._sale_places = None

        self._index: int = index



from quick_resto_API.quick_resto_objects.modules.warehouse.employee import Employee
from quick_resto_API.quick_resto_objects.quick_resto_object import QuickRestoObject


class Store(QuickRestoObject):
    @property
    def title(self) -> str:
        return self._title

    @property
    def description(self) -> str:
        return self._description

    @property
    def store_code(self) -> int:
        return self._store_code

    @property
    def lite_business(self) -> Employee:
        return self._lite_business

    def __init__(self, storeCode: str = None, title: str = None, liteBusiness: dict = None, description: str = "",
                 **kwargs):
        class_name: str = "ru.edgex.quickresto.modules.warehouse.store.Store"

        super().__init__(class_name=class_name, **kwargs)
        self._title: str = title
        self._description: str = description
        self._store_code: int = int(storeCode)
        
        if liteBusiness is not None:
            self._lite_business: Employee = Employee(**liteBusiness)
        else:
            self._lite_business = None



from quick_resto_API.operations_with_objects.operations_with_objects import OperationsWithObjects
from quick_resto_API.operations_with_objects.system_object import SystemObject
from quick_resto_API.quick_resto_api import QuickRestoApi
from quick_resto_API.quick_resto_objects.modules.warehouse.dish import Dish
from quick_resto_API.quick_resto_objects.modules.warehouse.dish_category import DishCategory


class DishOperations(SystemObject):
    def __init__(self, api: QuickRestoApi):
        self._operations_with_objects = OperationsWithObjects(api)

        self._module_name:str = "warehouse.nomenclature.dish"

    def get_list_of_dishes(self, ownerContextId: int = None, ownerContextClassName: str = None,
                           showDeleted: bool = False) -> list[Dish|DishCategory]:

        json_response = self._operations_with_objects.getList(self._module_name, ownerContextId, 
                                                                    ownerContextClassName, showDeleted).json()

        dishes:list[Dish|DishCategory] = list()

        for dish in json_response:
            if 'DishCategory' in dish['className']:
                dishes.append(DishCategory(**dish))
            elif 'Dish' in dish['className']:
                dishes.append(Dish(**dish))

        return dishes

    def get_tree_of_dishes(self, ownerContextId: int = None, ownerContextClassName: str = None,
                           showDeleted: bool = False) -> list[Dish|DishCategory]:

        json_response = self._operations_with_objects.getTree(self._module_name,
                                                              ownerContextId, ownerContextClassName, showDeleted).json()

        dishes:list[Dish|DishCategory] = list()

        for dish in json_response:
            if 'DishCategory' in dish['className']:
                dishes.append(DishCategory(**dish))
            elif 'Dish' in dish['className']:
                dishes.append(Dish(**dish))

        return dishes

    def get_dish_or_dish_category(self, objectId: int, objectRid: int = None) -> Dish | DishCategory:
        json_response = self._operations_with_objects.getObject(self._module_name, objectId, objectRid).json()

        if 'DishCategory' in json_response['className']:
            return DishCategory(**json_response)
        else:
            return Dish(**json_response)

    def get_dish_or_dish_category_with_subobjects(self, objectId: int, objectRid: int = None) -> Dish | DishCategory:
        json_response = self._operations_with_objects.getObjectWithSubobjects(self._module_name, objectId, objectRid).json()

        if 'DishCategory' in json_response['className']:
            return DishCategory(**json_response)
        else:
            return Dish(**json_response)

    def create_dish_or_dish_category(self, object: Dish | DishCategory,ownerContextId: int = None,
                                                ownerContextClassName: str = None, parentContextId: int = None,
                                                parentContextClassName: str = None) -> Dish | DishCategory:

        json_response = self._operations_with_objects.createObject(object, self._module_name, ownerContextId, 
                                                ownerContextClassName, parentContextId, parentContextClassName).json()

        if 'DishCategory' in json_response['className']:
            return DishCategory(**json_response)
        else:
            return Dish(**json_response)

    def update_dish_or_dish_category(self, object: Dish | DishCategory,ownerContextId: int = None,
                                                ownerContextClassName: str = None, parentContextId: int = None,
                                                parentContextClassName: str = None) -> Dish | DishCategory:

        json_response = self._operations_with_objects.updateObject(object, self._module_name, ownerContextId, 
                                                ownerContextClassName, parentContextId, parentContextClassName).json()

        if 'DishCategory' in json_response['className']:
            return DishCategory(**json_response)
        else:
            return Dish(**json_response)

    def remove_dish_or_dish_category(self, object: Dish | DishCategory,ownerContextId: int = None,
                                                ownerContextClassName: str = None, parentContextId: int = None,
                                                parentContextClassName: str = None) -> Dish | DishCategory:

        json_response = self._operations_with_objects.removeObject(object, self._module_name, ownerContextId, 
                                                ownerContextClassName, parentContextId, parentContextClassName).json()

        if 'DishCategory' in json_response['className']:
            return DishCategory(**json_response)
        else:
            return Dish(**json_response)

    def recover_dish_or_dish_category(self, object: Dish | DishCategory,ownerContextId: int = None,
                                                ownerContextClassName: str = None, parentContextId: int = None,
                                                parentContextClassName: str = None) -> Dish | DishCategory:

        json_response = self._operations_with_objects.recoverObject(object, self._module_name, ownerContextId, 
                                                ownerContextClassName, parentContextId, parentContextClassName).json()

        if 'DishCategory' in json_response['className']:
            return DishCategory(**json_response)
        else:
            return Dish(**json_response)


from quick_resto_API.quick_resto_objects.modules.warehouse.dish import Dish
from quick_resto_API.quick_resto_objects.quick_resto_object import QuickRestoObject

class AlcoholDictionary(QuickRestoObject):
    @property
    def deleted(self) -> bool:
        return self._deleted

    @property
    def egais_name(self) -> str:
        return self._egais_name

    @property
    def egais_size(self) -> int:
        return self._egais_size

    @property
    def egais_type_code(self) -> str:
        return self._egais_type_code

    @property
    def show_on_terminal(self) -> bool:
        return self._show_on_terminal

    @property
    def store_product(self) -> Dish:
        return self._store_product

    def __init__(self, deleted: bool = None, egaisName: str = None, egaisSize: int = None, egaisTypeCode: str = None,
                 showOnTerminal: bool = None, storeProduct: dict = None, **kwargs):
        class_name = "ru.edgex.quickresto.modules.alcohol.dictionary.AlcoholDictionary"

        super().__init__(class_name=class_name, **kwargs)

        self._deleted: bool = deleted
        self._egais_name: str = egaisName
        self._egais_size: int = egaisSize
        self._egais_type_code: str = egaisTypeCode
        self._show_on_terminal: bool = showOnTerminal

        if storeProduct is not None:
            self._store_product = Dish(**storeProduct)
        else:
            self._store_product = None



from enum import Enum

from quick_resto_API.quick_resto_objects.quick_resto_object import QuickRestoObject


class TableShape(Enum):
    RECT = 'rect'
    CIRCLE = 'circle'
    NONE = 'NONE'


StrToTableShape = {
    TableShape.RECT.value: TableShape.RECT,
    TableShape.CIRCLE.value: TableShape.CIRCLE
}

def convert_str_to_table_shape(table_shape: str) -> TableShape:
    if table_shape in StrToTableShape.keys():
        return StrToTableShape[table_shape]

    return TableShape.NONE


class Table(QuickRestoObject):
    @property
    def angle(self) -> int:
        return self._angle

    @property
    def deleted(self) -> bool:
        return self._deleted

    @property
    def x(self) -> int:
        return self._x

    @property
    def y(self) -> int:
        return self._y

    @property
    def width(self) -> int:
        return self._width

    @property
    def height(self) -> int:
        return self._height

    @property
    def is_busy(self) -> bool:
        return self._is_busy

    @property
    def item_title(self) -> str:
        return self._item_title

    @property
    def min_capacity(self) -> int:
        return self._min_capacity

    @property
    def max_capacity(self) -> int:
        return self._max_capacity

    @property
    def reservable(self) -> bool:
        return self._reservable

    @property
    def shape(self) -> TableShape:
        return self._shape

    @property
    def title(self) -> str:
        return self._title

    def __init__(self, angle: int = None, deleted: bool = None, width: int = None, height: int = None,
                 isBusy: bool = None, itemTitle: str = None,
                 minCapacity: int = None, maxCapacity: int = None, reservable: bool = None, shape: str = None,
                 title: str = None, x: int = None, y: int = None,
                 **kwargs):
        class_name = "ru.edgex.quickresto.modules.front.tablemanagement.table.Table"

        super().__init__(class_name=class_name, **kwargs)
        self._angle: int = angle
        self._deleted: bool = deleted
        self._x: int = x
        self._y: int = y
        self._width: int = width
        self._height: int = height
        self._is_busy: bool = isBusy
        self._item_title: str = itemTitle
        self._min_capacity: int = minCapacity
        self._max_capacity: int = maxCapacity
        self._reservable: bool = reservable
        self._title: str = title
        self._shape: TableShape = convert_str_to_table_shape(shape)



from quick_resto_API.quick_resto_objects.quick_resto_object import QuickRestoObject


class Organization(QuickRestoObject):
    @property
    def short_name(self) -> str:
        return self._short_name

    @property
    def address(self) -> str:
        return self._address

    @property
    def delivery_address(self) -> str:
        return self._delivery_address

    @property
    def egais_status(self) -> str:
        return self._egais_status

    @property
    def egais_activity_status(self) -> str:
        return self._egais_activity_status

    @property
    def full_name(self) -> str:
        return self._full_name

    def __init__(self, shortName: str = None, address: str = None, deliveryAddress: str = None, egaisStatus: str = None,
                 egaisActivityStatus: str = None, fullName: str = None, **kwargs):
        class_name = "ru.edgex.quickresto.modules.warehouse.providers.Organization"

        super().__init__(class_name=class_name, **kwargs)

        self._short_name: str = shortName
        self._address: str = address
        self._delivery_address: str = deliveryAddress
        self._egais_status: str = egaisStatus
        self._egais_activity_status: str = egaisActivityStatus
        self._full_name: str = fullName



from quick_resto_API.quick_resto_objects.modules.alcohol.alcohol_dictionary import AlcoholDictionary
from quick_resto_API.quick_resto_objects.quick_resto_object import QuickRestoObject


class AlcoholReport(QuickRestoObject):
    @property
    def deleted(self) -> bool:
        return self._deleted

    @property
    def tap_time(self) -> int:
        return self._tap_time

    @property
    def count(self) -> int:
        return self._count

    @property
    def alcohol_dictionary(self) -> AlcoholDictionary:
        return self._alcohol_dictionary

    def __init__(self, deleted: bool = None, tapTime: int = None, count: int = None, alcoholDictionary: dict = None,
                 **kwargs):
        class_name = "ru.edgex.quickresto.modules.alcohol.report.AlcoholReport"

        super().__init__(class_name=class_name, **kwargs)

        self._deleted: bool = deleted
        self._tap_time: int = tapTime
        self._count: int = count
        
        if alcoholDictionary is not None: 
            self._alcohol_dictionary = AlcoholDictionary(**alcoholDictionary)
        else:
            self._alcohol_dictionary = None



from quick_resto_API.quick_resto_objects.modules.warehouse.cooking_place import CookingPlace
from quick_resto_API.quick_resto_objects.modules.warehouse.sale_place import SalePlace
from quick_resto_API.quick_resto_objects.quick_resto_object import QuickRestoObject


class DishSale(QuickRestoObject):
    @property
    def sale_place(self):
        return self._sale_place

    @property
    def cooking_place(self):
        return self._cooking_place

    def __init__(self, cookingPlace: dict = None, price: float = None, salePlace: dict = None, **kwargs):
        class_name = "ru.edgex.quickresto.modules.warehouse.nomenclature.dish.dishSale.DishSale"

        super().__init__(class_name=class_name, **kwargs)
        self._price: float = price

        if salePlace is not None: 
            self._sale_place: SalePlace = SalePlace(**salePlace)
        else:
            self._sale_place = None

        if cookingPlace is not None: 
            self._cooking_place: CookingPlace = CookingPlace(**cookingPlace)
        else:
            self._cooking_place = None



from quick_resto_API.quick_resto_objects.modules.core.measure_unit import MeasureUnit
from quick_resto_API.quick_resto_objects.modules.core.store_item_tag import StoreItemTag
from quick_resto_API.quick_resto_objects.modules.warehouse.dish_sale import DishSale
from quick_resto_API.quick_resto_objects.quick_resto_object import QuickRestoObject


class Modifier(QuickRestoObject):
    @property
    def name(self) -> str:
        return self._name

    @property
    def deleted(self) -> bool:
        return self._deleted

    @property
    def article(self) -> int:
        return self._article

    @property
    def with_dish(self) -> bool:
        return self._with_dish

    @property
    def measure_unit(self) -> MeasureUnit:
        return self._measure_unit

    @property
    def title_in_price(self) -> str:
        return self._titleInPrice

    @property
    def item_title(self) -> bool:
        return self._item_title

    @property
    def min_value(self) -> int:
        return self._min_value

    @property
    def max_value(self) -> int:
        return self._max_value

    @property
    def modifier_sales(self) -> list:
        return self._modifier_sales

    @property
    def exclude_discount(self) -> bool:
        return self._exclude_discount

    @property
    def exclude_markup(self) -> bool:
        return self._exclude_markup

    @property
    def store_quantity_kg(self) -> float:
        return self._store_quantity_kg

    @property
    def default_value(self) -> int:
        return self._default_value

    @property
    def minimal_price(self) -> float:
        return self._minimal_price

    @property
    def base_price_in_list(self) -> float:
        return self._base_price_in_list

    @property
    def pack(self) -> float:
        return self._pack

    @property
    def recipe(self) -> str:
        return self._recipe

    @property
    def price(self) -> float:
        return self._price

    @property
    def store_item_tag(self) -> StoreItemTag:
        return self._store_item_tag

    @property
    def parent_id(self) -> int:
        return self._parent_id

    @property
    def parent_item(self) -> dict:
        return self._parent_item

    def __init__(self, name: str = None, measureUnit: dict = None, color: str = None, displayOnTerminal: bool = None,
                 minValue: int = None, maxValue: int = None, withDish: bool = None,
                 modifierSales: list = None, excludeDiscount: bool = None, excludeMarkup: bool = None,
                 article: int = None, itemTitle: str = None, parentId:int = None, parentItem:dict = None,
                 titleInPrice: str = None, storeQuantityKg: float = None, defaultValue: int = None,
                 basePriceInList: float = None, pack: float = None, recipe: str = None,
                 price: float = None, minimalPrice: float = None, deleted: bool = None, storeItemTag=None, **kwargs):
        class_name = "ru.edgex.quickresto.modules.warehouse.nomenclature.mods.Modifier"

        super().__init__(class_name=class_name, **kwargs)

        self._name = name
        self._deleted = deleted
        self._measure_unit = measureUnit
        self._color = color
        self._display_on_terminal = displayOnTerminal
        self._min_value = minValue
        self._max_value = maxValue
        self._with_dish = withDish

        if modifierSales is not None:
            self._modifier_sales: list = [DishSale(**dish_sale) for dish_sale in modifierSales]
        else:
            self._modifier_sales = None

        self._item_title = itemTitle
        self._article = article
        self._titleInPrice = titleInPrice
        self._exclude_discount = excludeDiscount
        self._exclude_markup = excludeMarkup
        self._store_quantity_kg = storeQuantityKg
        self._default_value = defaultValue
        self._minimal_price = minimalPrice
        self._base_price_in_list = basePriceInList
        self._pack = pack
        self._recipe = recipe
        self._price = price

        if storeItemTag is not None:
            self._store_item_tag = StoreItemTag(**storeItemTag)
        else:
            self._store_item_tag = None

        self._parent_id = parentId
        self._parent_item = parentItem
        


from enum import Enum

from quick_resto_API.quick_resto_objects.quick_resto_object import QuickRestoObject


class TokenType(Enum):
    NONE = "NONE"
    PIN = "pin"
    CARD = "card"
    PHONE = "phone"
    QR_GUEST = "qrGuest"


class EntryType(Enum):
    NONE = "NONE"
    BAR_CODE = "barCode"
    TRACK_CODE = "trackCode"
    QR_CODE = "qrCode"
    MANUAL = "manual"


StrToCrmCustomerTokenType = {
    TokenType.PIN.value: TokenType.PIN,
    TokenType.CARD.value: TokenType.CARD,
    TokenType.PHONE.value: TokenType.PHONE,
    TokenType.QR_GUEST.value: TokenType.QR_GUEST
}

StrToCrmCustomerEntryType = {
    EntryType.BAR_CODE.value: EntryType.BAR_CODE,
    EntryType.TRACK_CODE.value: EntryType.TRACK_CODE,
    EntryType.QR_CODE.value: EntryType.QR_CODE,
    EntryType.MANUAL.value: EntryType.MANUAL
}


def convert_str_to_crm_token_type(crm_token_type: str) -> TokenType:
    crm_token_type = crm_token_type.upper()

    if crm_token_type in TokenType.__members__.keys():
        return TokenType.__members__[crm_token_type]

    return TokenType.NONE


def convert_str_to_crm_entry_type(crm_entry_type: str) -> EntryType:
    crm_entry_type = crm_entry_type.upper()

    if crm_entry_type in EntryType.__members__.keys():
        return EntryType.__members__[crm_entry_type]

    return EntryType.NONE


class CustomerToken(QuickRestoObject):
    @property
    def token_type(self) -> TokenType:
        return self._token_type

    @property
    def entry(self) -> EntryType:
        return self._entry

    @property
    def key(self) -> str:
        return self._key

    def __init__(self, entry: str = None, key: str = None, type: str = None, **kwargs):
        class_name = "ru.edgex.quickresto.modules.crm.customer.CrmToken"

        super().__init__(class_name=class_name, **kwargs)
        self._entry: EntryType = convert_str_to_crm_entry_type(entry)
        self._key: str = key
        self._token_type: TokenType = convert_str_to_crm_token_type(type)



from enum import Enum

from quick_resto_API.quick_resto_objects.modules.warehouse.store import Store
from quick_resto_API.quick_resto_objects.quick_resto_object import QuickRestoObject


class CustomerType(Enum):
    PARTNER = "PARTNER"
    GUEST = "GUEST"
    EMPLOYEE = "EMPLOYEE"
    NONE = "NONE"


StrToCustomerType = {
    CustomerType.PARTNER.value: CustomerType.PARTNER,
    CustomerType.GUEST.value: CustomerType.GUEST,
    CustomerType.EMPLOYEE.value: CustomerType.EMPLOYEE
}


def convert_str_to_customer_type(value: str) -> CustomerType:
    if value in StrToCustomerType.keys():
        return StrToCustomerType[value]

    return CustomerType.NONE


class OutgoingInvoice(QuickRestoObject):
    @property
    def document_number(self) -> str:
        return self._document_number

    @property
    def invoice_date(self) -> str:
        return self._invoice_date

    @property
    def customer_type(self) -> CustomerType:
        return self._customer_type

    @property
    def store(self) -> Store:
        return self._store

    @property
    def processed(self) -> bool:
        return self._processed

    @property
    def subject(self) -> dict:
        return self._subject

    @property
    def operator(self) -> dict:
        return self._operator

    @property
    def guest(self) -> dict:
        return self._guest

    @property
    def total_sum(self) -> float:
        return self._total_sum

    @property
    def total_sum_wo_nds(self) -> float:
        return self._total_sum_wo_nds

    @property
    def total_nds(self) -> float:
        return self._total_nds

    @property
    def cost_price_sum(self) -> float:
        return self._cost_price_sum

    @property
    def reason(self) -> str:
        return self._reason

    @property
    def comment(self) -> str:
        return self._comment

    @property
    def total_amount(self) -> float:
        return self._total_amount

    def __init__(self, documentNumber: str = None, invoiceDate: str = None, customerType: str = None,
                 store: dict = None, processed: bool = None, totalSum: float = None,
                 totalSumWoNds: float = None, totalNds: float = None, costPriceSum: float = None, reason: str = None,
                 comment: str = None, totalAmount: float = None,
                 subject: dict = None, guest: dict = None, operator: dict = None, **kwargs):
        class_name = "ru.edgex.quickresto.modules.warehouse.documents.outgoing.OutgoingInvoice"

        super().__init__(class_name=class_name, **kwargs)

        self._document_number: str = documentNumber
        self._invoice_date: str = invoiceDate

        self._processed: bool = processed
        self._subject: dict = subject
        self._operator: dict = operator
        self._guest: dict = guest
        self._total_sum: float = totalSum
        self._total_sum_wo_nds: float = totalSumWoNds
        self._total_nds: float = totalNds
        self._cost_price_sum: float = costPriceSum
        self._reason: str = reason
        self._comment: str = comment
        self._total_amount: float = totalAmount

        if store is not None: 
            self._store = Store(**store)
        else:
            self._store = None

        if customerType is not None:
            self._customer_type: CustomerType = convert_str_to_customer_type(customerType)
        else:
            self._customer_type = None



from datetime import date

from quick_resto_API.quick_resto_api import QuickRestoApi
from quick_resto_API.quick_resto_objects.modules.crm.account_type import AccountType
from quick_resto_API.quick_resto_objects.modules.crm.account_balance import AccountBalance
from quick_resto_API.quick_resto_objects.modules.crm.customer import CrmCustomer
from quick_resto_API.quick_resto_objects.modules.crm.customer_token import CustomerToken

class QuickRestoInterface:
    @property
    def api(self):
        return self._api

    def __init__(self, login: str, password: str, use_https: bool = True, layer: str = "quickresto.ru"):
        self._api: QuickRestoApi = QuickRestoApi(login, password, use_https, layer)
        self._ping()

    def _ping(self) -> None:
        # TODO: add good error message for this case
        self._api.get("ping")

    def crm_search_client(self, search: str) -> CrmCustomer:
        json_data = {
            "search": search
        }

        json_response = self._api.post("bonuses/filterCustomers", json_data=json_data).json()

        return CrmCustomer(**json_response)

    def crm_get_customer_info(self, customerToken: CustomerToken) -> CrmCustomer:
        json_data = {
            "customerToken": customerToken.get_json_object
        }

        json_response = self._api.post("bonuses/customerInfo", json_data=json_data).json()

        return CrmCustomer(**json_response)

    def crm_get_client_balance(self, customerToken: CustomerToken, accountType: AccountType) -> AccountBalance:
        json_data = {
            "customerToken": customerToken.get_json_object(),
            "accountType": accountType.get_json_object()
        }

        json_response = self._api.post("bonuses/balance", json_data=json_data).json()

        return AccountBalance(**json_response)

    def crm_get_operation_history(self, customerToken: CustomerToken) -> dict:
        json_data = {
            "customerToken": customerToken.get_json_object()
        }

        json_response = self._api.post("bonuses/operationHistory", json_data=json_data).json()

        return json_response

    def crm_create_customer(self, customer: CrmCustomer) -> CrmCustomer:
        json_data = customer.get_json_object()

        json_response = self._api.post("bonuses/createCustomer", json_data=json_data).json()

        return CrmCustomer(**json_response)

    def crm_depit_hold(self, customerToken: CustomerToken, amount: int, accountType: AccountType, date: date = None,
                       precheck: int = None) -> dict:
        json_data = {
            "customerToken": customerToken.get_json_object(),
            "date": date,
            "precheck": precheck,
            "amount": amount,
            "accountType": accountType.get_json_object(),
        }

        json_response = self._api.post("bonuses/debitHold", json_data=json_data).json()

        return json_response

    def crm_credit_hold(self, customerToken: CustomerToken, accountType: AccountType, amount: int, date: date = None,
                        precheck: int = None) -> dict:
        json_data = {
            "customerToken": customerToken.get_json_object(),
            "date": date,
            "precheck": precheck,
            "amount": amount,
            "accountType": accountType.get_json_object(),
        }

        json_response = self._api.post("bonuses/creditHold", json_data=json_data).json()

        return json_response

    def crm_reverse(self, customerToken: CustomerToken, accountType: AccountType, amount: int, bonusTransactionId: int,
                    date: date = None, precheck: int = None) -> dict:
        json_data = {
            "customerToken": customerToken.get_json_object(),
            "date": date,
            "precheck": precheck,
            "amount": amount,
            "accountType": accountType.get_json_object(),
            "bonusTransactionId": bonusTransactionId
        }

        json_response = self._api.post("bonuses/reverse", json_data=json_data).json()

        return json_response



import json

import requests
from requests.auth import HTTPBasicAuth


class QuickRestoApiException(BaseException):
    @property
    def message(self) -> str:
        return self._message

    @property
    def status(self) -> int:
        return self._status

    @property
    def response(self) -> requests.Response:
        return self._response

    def __init__(self, response: requests.Response):
        self._response: requests.Response = response
        self._status: int = response.status_code
        self._message: str = self._prettify_message()
        super().__init__(f"Http code is {self.status}.\nMessage from server:\n{self.message}")

    def _prettify_message(self) -> str:
        try:
            json_data = self.response.json()
            return json.dumps(json_data, sort_keys=True, ensure_ascii=False, indent=4)
        except requests.exceptions.JSONDecodeError:
            return self.response.text


class QuickRestoApi:
    @property
    def login(self) -> str:
        return self._login

    @property
    def base_url(self) -> str:
        return self._base_url

    @property
    def headers(self) -> dict:
        return self._headers

    def __init__(self, login: str, password: str, use_https: bool = True, layer: str = "quickresto.ru"):
        self._login: str = login
        self._password: str = password
        self._use_https = use_https
        self._layer = layer
        self._base_url: str = f'http{"s" * use_https}://{login}.{layer}/platform/online/'
        self._auth_data: HTTPBasicAuth = HTTPBasicAuth(self._login, self._password)
        self._headers: dict = {
            "Content-type": "application/json",
            "Connection": "keep-alive"
        }

    def post(self, url: str, parameters: dict = None, json_data: dict = None) -> requests.Response:
        response = requests.post(
            url=self.base_url + url,
            headers=self.headers,
            json=json_data,
            auth=self._auth_data,

            params=parameters
        )
        self._check_response(response)

        return response

    def get(self, url: str, parameters: dict = None, json_data: dict = None) -> requests.Response:
        response = requests.get(
            url=self.base_url + url,
            headers=self.headers,
            json=json_data,
            auth=self._auth_data,

            params=parameters
        )
        self._check_response(response)

        return response

    def get_hashes(self) -> dict:
        response = requests.get(
            url= f'http{"s" * self._use_https}://{self._login}.{self._layer}/wt/hashes',
            headers=self.headers,
            auth=self._auth_data,
        )

        return response.json()

    @staticmethod
    def _check_response(response: requests.Response) -> None:
        if response.status_code // 100 != 2:
            raise QuickRestoApiException(response)



from quick_resto_API.quick_resto_objects.modules.core.measure_unit import MeasureUnit
from quick_resto_API.quick_resto_objects.quick_resto_object import QuickRestoObject

class PackingUnit(QuickRestoObject):
    @property
    def version(self) -> int:
        return self._version

    @property
    def name(self) -> str:
        return self._name

    @property
    def parent_ratio(self) -> float:
        return self._parent_ratio

    @property
    def parent_unit(self) -> MeasureUnit:
        return self._parent_unit

    def __init__(self, version: int = None, name: str = None, parentRatio: float = None, parentUnit: dict = None,
                 **kwargs):
        class_name = "ru.edgex.quickresto.modules.core.dictionaries.packingunits.PackingUnit"

        super().__init__(class_name=class_name, **kwargs)

        self._version: int = version
        self._name: str = name
        self._parent_ratio: float = parentRatio

        if parentUnit is not None: 
            self._parent_unit = MeasureUnit(**parentUnit)
        else:
            self._parent_unit = None



from quick_resto_API.quick_resto_objects.platform.right_link import RightLink
from quick_resto_API.quick_resto_objects.quick_resto_object import QuickRestoObject


class Role(QuickRestoObject):
    @property
    def system_role(self) -> str:
        return self._system_role

    @property
    def title(self) -> str:
        return self._title

    @property
    def back_office_user(self) -> bool:
        return self._back_office_user

    @property
    def front_office_user(self) -> bool:
        return self._front_office_user

    @property
    def courier_user(self) -> bool:
        return self._courier_user

    @property
    def right_links(self) -> list:
        return self._right_links

    def __init__(self, systemRole: str = None, title: str = None, backOfficeUser: bool = None,
                 frontOfficeUser: bool = None, courierUser: bool = None,
                 rightLinks: list = None, **kwargs):
        class_name = "ru.edgex.platform.service.user.Role"

        super().__init__(class_name=class_name, **kwargs)

        self._system_role: str = systemRole
        self._title: str = title
        self._back_office_user: bool = backOfficeUser
        self._front_office_user: bool = frontOfficeUser
        self._courier_user: bool = courierUser

        if rightLinks is not None: 
            self._right_links: list = [RightLink(**value) for value in rightLinks]
        else:
            self._right_links = None



from quick_resto_API.quick_resto_objects.modules.front.table_scheme import TableScheme
from quick_resto_API.quick_resto_objects.modules.front.terminal import Terminal
from quick_resto_API.quick_resto_objects.modules.front.kkm_terminal import KkmTerminal
from quick_resto_API.quick_resto_objects.modules.warehouse.cooking_place import CookingPlace
from quick_resto_API.quick_resto_objects.quick_resto_object import QuickRestoObject


class SalePlace(QuickRestoObject):
    @property
    def automatic_encashment(self) -> bool:
        return self._automatic_encashment

    @property
    def can_hold_table_orders(self) -> bool:
        return self._can_hold_table_orders

    @property
    def default_cooking_place(self) -> CookingPlace:
        return self._default_cooking_place

    @property
    def max_table_order_number_enable(self) -> bool:
        return self._max_table_order_number_enable

    @property
    def open_cash_box_on_guest_tickets(self) -> bool:
        return self._open_cash_box_on_guest_tickets

    @property
    def open_cash_box_on_prechecks(self) -> bool:
        return self._open_cash_box_on_prechecks

    @property
    def open_cash_box_on_reports(self) -> bool:
        return self._open_cash_box_on_reports

    @property
    def print_guest_ticket(self) -> bool:
        return self._print_guest_ticket

    @property
    def print_kitchen_ticket(self) -> bool:
        return self._print_kitchen_ticket

    @property
    def services(self) -> list:
        return self._services

    @property
    def title(self) -> str:
        return self._title

    @property
    def table(self) -> TableScheme:
        return self._table

    @property
    def backup_prechecks_target_device(self) -> Terminal:
        return self._backup_prechecks_target_device

    @property
    def kkm_organization1(self) -> KkmTerminal:
        return self._kkm_organization1

    @property
    def kkm_organization2(self) -> KkmTerminal:
        return self._kkm_organization2

    @property
    def prechecks_target_device(self) -> Terminal:
        return self._prechecks_target_device

    @property
    def report_target_device(self) -> Terminal:
        return self._report_target_device

    def __init__(self, automaticEncashment: bool = None, canHoldTableOrders: bool = None,
                 defaultCookingPlace: dict = None,
                 maxTableOrderNumberEnable: bool = None, openCashBoxOnGuestTickets: bool = None,
                 openCashBoxOnPrechecks: bool = None,
                 openCashBoxOnReports: bool = None, printGuestTicket: bool = None, printKitchenTicket: bool = None,
                 services: list = None,
                 tableScheme: dict = None, title: str = None, backupPrechecksTargetDevice: dict = None,
                 kkmOrganization1: dict = None, kkmOrganization2: dict = None,
                 prechecksTargetDevice: dict = None, reportTargetDevice: dict = None, **kwargs):
        class_name: str = "ru.edgex.quickresto.modules.warehouse.nomenclature.sale_place.SalePlace"

        super().__init__(class_name=class_name, **kwargs)
        self._automatic_encashment: bool = automaticEncashment
        self._can_hold_table_orders: bool = canHoldTableOrders

        if defaultCookingPlace is not None: 
            self._default_cooking_place: CookingPlace = CookingPlace(**defaultCookingPlace)
        else:
            self._default_cooking_place = None

        self._max_table_order_number_enable: bool = maxTableOrderNumberEnable
        self._open_cash_box_on_guest_tickets: bool = openCashBoxOnGuestTickets
        self._open_cash_box_on_prechecks: bool = openCashBoxOnPrechecks
        self._open_cash_box_on_reports: bool = openCashBoxOnReports
        self._print_guest_ticket: bool = printGuestTicket
        self._print_kitchen_ticket: bool = printKitchenTicket
        self._services: list = services
        self._title: str = title

        if tableScheme is not None: 
            self._table: TableScheme = TableScheme(**tableScheme)
        else:
            self._table = None

        if backupPrechecksTargetDevice is not None: 
            self._backup_prechecks_target_device = Terminal(**backupPrechecksTargetDevice)
        else:
            self._backup_prechecks_target_device = None

        if kkmOrganization1 is not None: 
            self._kkm_organization1 = KkmTerminal(**kkmOrganization1)
        else:
            self._kkm_organization1 = None

        if kkmOrganization2 is not None: 
            self._kkm_organization2 = KkmTerminal(**kkmOrganization2)
        else:
            self._kkm_organization2 = None

        if prechecksTargetDevice is not None: 
            self._prechecks_target_device = Terminal(**prechecksTargetDevice)
        else:
            self._prechecks_target_device = None

        if reportTargetDevice is not None: 
            self._report_target_device = Terminal(**reportTargetDevice)
        else:
            self._report_target_device = None



from quick_resto_API.quick_resto_objects.modules.core.store_item_tag import StoreItemTag
from quick_resto_API.quick_resto_objects.modules.crm.account_type import AccountType
from quick_resto_API.quick_resto_objects.modules.crm.group import Group
from quick_resto_API.quick_resto_objects.modules.crm.markup import Day
from quick_resto_API.quick_resto_objects.modules.warehouse.dish import Dish
from quick_resto_API.quick_resto_objects.modules.warehouse.dish_category import DishCategory
from quick_resto_API.quick_resto_objects.quick_resto_object import QuickRestoObject

class BonusProgram(QuickRestoObject):
    @property
    def name(self) -> str:
        return self._name

    @property
    def deleted(self) -> bool:
        return self._deleted

    @property
    def days(self) -> list:
        return self._days

    @property
    def groups(self) -> list:
        return self._groups

    @property
    def start_date(self) -> str:
        return self._start_date

    @property
    def end_date(self) -> str:
        return self._end_date

    @property
    def acc_value(self) -> float:
        return self._acc_value

    @property
    def account_type(self) -> AccountType:
        return self._account_type

    @property
    def do_not_accumulate_while_redeeming(self) -> bool:
        return self._do_not_accumulate_while_redeeming

    @property
    def greeting_bonus(self) -> float:
        return self._greeting_bonus

    @property
    def birthday_bonus(self) -> float:
        return self._birthday_bonus

    @property
    def categories(self) -> list:
        return self._categories

    @property
    def dishes(self) -> list:
        return self._dishes

    @property
    def tags(self) -> list:
        return self._tags

    def __init__(self, name: str = None, deleted: bool = None, days: list = None, groups: list = None,
                 startDate: str = None, endDate: str = None,
                 accValue: float = None, accountType: dict = None, doNotAccumulateWhileRedeeming: bool = None,
                 greetingBonus: float = None,
                 birthdayBonus: float = None, categories: list = None, dishes: list = None, tags: list = None,
                 **kwargs):
        class_name = "ru.edgex.quickresto.modules.crm.settings.bonus.BonusProgram"

        super().__init__(class_name=class_name, **kwargs)

        self._name: str = name
        self._deleted: bool = deleted

        if days is not None: 
            self._days: list = [Day(**day) for day in days]
        else:
            self._days = None 

        if groups is not None: 
            self._groups: list = [Group(**group) for group in groups]
        else:
            self._groups = None

        self._start_date: str = startDate
        self._end_date: str = endDate
        self._acc_value: float = accValue

        if accountType is not None: 
            self._account_type: dict = AccountType(**accountType)
        else:
            self._account_type = None

        self._do_not_accumulate_while_redeeming: bool = doNotAccumulateWhileRedeeming
        self._greeting_bonus: float = greetingBonus
        self._birthday_bonus: float = birthdayBonus

        if categories is not None: 
            self._categories = [DishCategory(**category) for category in categories]
        else:
            self._categories = None

        if dishes is not None: 
            self._dishes: list = [Dish(**dish) for dish in dishes]
        else:
            self._dishes = None

        if tags is not None: 
            self._tags: list = [StoreItemTag(**tag) for tag in tags]
        else:
            self._tags = None



from quick_resto_API.quick_resto_objects.quick_resto_object import QuickRestoObject


class Employee(QuickRestoObject):
    @property
    def name(self) -> str:
        return self._name

    @property
    def nds(self) -> bool:
        return self._nds

    @property
    def short_name(self) -> str:
        return self._short_name

    def __init__(self, nds: bool = None, shortName: str = None, name: str = "", **kwargs):
        class_name: str = "ru.edgex.quickresto.modules.warehouse.store.employee.Employee"

        super().__init__(class_name=class_name, **kwargs)
        self._name: str = name
        self._nds: bool = nds
        self._short_name: str = shortName



def to_snake_case(given_str: str) -> str:
    result: list = []

    for char in given_str:
        if char.isupper():
            result.append('_')
            result.append(char.lower())
        else:
            result.append(char)

    return "".join(result)

def to_camel_case(given_str: str) -> str:
    if len(given_str) == 0:
        return ""

    if given_str[0] == '_':
        given_str = given_str[1:]

    result: list = []
    next_uppercase: bool = False

    for char in given_str:
        if char == '_':
            next_uppercase = True
        elif next_uppercase:
            next_uppercase = False
            result.append(char.upper())
        else:
            result.append(char)

    return "".join(result)


from enum import Enum

from quick_resto_API.quick_resto_objects.modules.crm.contact_method import ContactMethod
from quick_resto_API.quick_resto_objects.modules.crm.customer_account import CustomerAccount
from quick_resto_API.quick_resto_objects.modules.crm.customer_token import CustomerToken
from quick_resto_API.quick_resto_objects.modules.crm.group import Group
from quick_resto_API.quick_resto_objects.quick_resto_object import QuickRestoObject


class CrmCustomerType(Enum):
    NONE = "NONE"
    CUSTOMER = 'customer'


class CrmCustomerSex(Enum):
    NONE = "NONE"
    MALE = "male"
    FEMALE = "female"


StrToCrmCustomerType = {
    CrmCustomerType.CUSTOMER.value: CrmCustomerType.CUSTOMER
}

StrToCrmCustomerSex = {
    CrmCustomerSex.MALE.value: CrmCustomerSex.MALE,
    CrmCustomerSex.FEMALE.value: CrmCustomerSex.FEMALE
}


def convert_str_to_crm_customer_type(crm_customer_type: str) -> CrmCustomerType:
    if crm_customer_type in StrToCrmCustomerType.keys():
        return StrToCrmCustomerType[crm_customer_type]

    return CrmCustomerType.NONE


def convert_str_to_crm_customer_sex(crm_customer_sex: str) -> CrmCustomerSex:
    if crm_customer_sex in StrToCrmCustomerSex.keys():
        return StrToCrmCustomerSex[crm_customer_sex]

    return CrmCustomerSex.NONE


class CrmCustomer(QuickRestoObject):
    @property
    def accounts(self) -> list:
        return self._accounts

    @property
    def addresses(self) -> list:
        return self._addresses

    @property
    def contact_methods(self) -> list:
        return self._contact_methods

    @property
    def customer_guid(self) -> str:
        return self._customer_guid

    @property
    def first_name(self) -> str:
        return self._first_name

    @property
    def middle_name(self) -> str:
        return self._middle_name

    @property
    def last_name(self) -> str:
        return self._last_name

    @property
    def sex(self) -> CrmCustomerSex:
        return self._sex

    @property
    def tokens(self) -> list:
        return self._tokens

    @property
    def comment(self) -> str:
        return self._comment

    @property
    def date_of_birth(self) -> str:
        return self._date_of_birth

    @property
    def customer_type(self) -> CrmCustomerType:
        return self._customer_type

    def __init__(self, accounts: list = None, addresses: list = None, contactMethods: list = None,
                 customerGuid: str = None,
                 firstName: str = None, lastName: str = None, tokens: list = None, type: str = None,
                 customerGroup: dict = None, comment: str = "",
                 middleName: str = "", sex: str = "", dateOfBirth: str = "", **kwargs):
        class_name: str = 'ru.edgex.quickresto.modules.crm.customer.CrmCustomer'

        super().__init__(class_name=class_name, **kwargs)
        if (accounts != None): self._accounts: list = [CustomerAccount(**account) for account in accounts]
        self._addresses: list = addresses
        if (contactMethods != None): self._contact_methods: list = [ContactMethod(**contact_method) for contact_method
                                                                    in contactMethods]
        self._customer_guid: str = customerGuid
        self._first_name: str = firstName
        self._middle_name: str = middleName
        self._last_name: str = lastName
        self._sex: CrmCustomerSex = convert_str_to_crm_customer_sex(sex)
        self._comment: str = comment
        self._date_of_birth: str = dateOfBirth

        if tokens is not None: 
            self._tokens: list = [CustomerToken(**token) for token in tokens]
        else:
            self._tokens = None

        self._customer_type: CrmCustomerType = convert_str_to_crm_customer_type(type)

        if customerGroup is not None: 
            self._group: Group = Group(**customerGroup)
        else:
            self._group = None



from quick_resto_API.quick_resto_objects.quick_resto_object import QuickRestoObject


class Business(QuickRestoObject):
    @property
    def account(self) -> str:
        return self._account

    @property
    def address(self) -> str:
        return self._address

    @property
    def bank_id(self) -> str:
        return self._bank_id

    @property
    def bank_name(self) -> str:
        return self._bank_name

    @property
    def correspondent_account(self) -> str:
        return self._correspondent_account

    @property
    def delivery_address(self) -> str:
        return self._delivery_address

    @property
    def deleted(self) -> bool:
        return self._deleted

    @property
    def inn_code(self) -> str:
        return self._inn_code

    @property
    def kpp_code(self) -> str:
        return self._kpp_code

    @property
    def name(self) -> str:
        return self._name

    @property
    def okpo_code(self) -> str:
        return self._okpo_code

    @property
    def short_name(self) -> str:
        return self._short_name

    def __init__(self, account: str = None, address: str = None, bankId: str = None, bankName: str = None,
                 correspondentAccount: str = None,
                 deliveryAddress: str = None, deleted: bool = None, innCode: str = None, kppCode: str = None,
                 name: str = None, okpoCode: str = None,
                 shortName: str = None, **kwargs):
        class_name = "ru.edgex.quickresto.modules.core.company.businesses.Business"

        super().__init__(class_name=class_name, **kwargs)

        self._account: str = account
        self._address: str = address
        self._bank_id: str = bankId
        self._bank_name: str = bankName
        self._correspondent_account: str = correspondentAccount
        self._delivery_address: str = deliveryAddress
        self._deleted: bool = deleted
        self._inn_code: str = innCode
        self._kpp_code: str = kppCode
        self._name: str = name
        self._okpo_code: str = okpoCode
        self._short_name: str = shortName



from enum import Enum

from quick_resto_API.quick_resto_objects.quick_resto_object import QuickRestoObject


class SystemUnit(Enum):
    ITEM = "ITEM",
    KG = "KG",
    L = "L",
    RATION = "RATION",
    Custom = "Custom"


StrToSystemUnit = {
    SystemUnit.ITEM.value: SystemUnit.ITEM,
    SystemUnit.KG.value: SystemUnit.KG,
    SystemUnit.L.value: SystemUnit.L,
    SystemUnit.RATION.value: SystemUnit.RATION,
    SystemUnit.Custom.value: SystemUnit.Custom
}


def convert_str_to_system_unit(value: str) -> SystemUnit:
    if value in StrToSystemUnit.keys():
        return StrToSystemUnit[value]

    return SystemUnit.Custom


class MeasureUnit(QuickRestoObject):
    @property
    def name(self) -> str:
        return self._name

    @property
    def parent_ratio(self) -> float:
        return self._parent_ratio

    @property
    def system_unit(self) -> SystemUnit:
        return self._system_unit

    @property
    def full_name(self) -> str:
        return self._full_name

    @property
    def is_main_unit(self) -> bool:
        return self._is_main_unit

    @property
    def code(self) -> str:
        return self._code

    def __init__(self, name: str = None, fullName: str = None, isMainUnit: bool = None, parentRatio: float = None,
                 code: str = None,
                 systemUnit: str = "ITEM", **kwargs):
        class_name = "ru.edgex.quickresto.modules.core.dictionaries.measureunits.MeasureUnit"

        super().__init__(class_name=class_name, **kwargs)

        self._name: str = name
        self._parent_ratio: float = parentRatio
        self._system_unit = convert_str_to_system_unit(systemUnit)
        self._full_name: str = fullName
        self._is_main_unit: bool = isMainUnit
        self._code: str = code



from quick_resto_API.quick_resto_objects.quick_resto_object import QuickRestoObject


class AccountType(QuickRestoObject):
    @property
    def name(self) -> str:
        return self._name

    @property
    def account_guid(self) -> str:
        return self._account_guid

    @property
    def deleted(self) -> float:
        return self._deleted

    @property
    def max_usage(self) -> float:
        return self._max_usage

    @property
    def reset_period(self) -> float:
        return self._reset_period

    def __init__(self, accountGuid: str = None, deleted: bool = None, maxUsage: float = None, name: str = None,
                 resetPeriod: float = None, **kwargs):
        class_name = "ru.edgex.quickresto.modules.crm.accounting.account.account_balance.AccountType"

        super().__init__(id=0, class_name=class_name, **kwargs)
        self._name: str = name
        self._account_guid: str = accountGuid
        self._deleted: bool = deleted
        self._max_usage: float = maxUsage
        self._reset_period: float = resetPeriod



from quick_resto_API.quick_resto_api import QuickRestoApi
from quick_resto_API.quick_resto_objects.quick_resto_object import QuickRestoObject

class OperationsWithObjects:
    def __init__(self, quick_resto_api: QuickRestoApi):
        self._api: QuickRestoApi = quick_resto_api

    def getList(self, moduleName: str, ownerContextId: int = None,
                ownerContextClassName: str = None, showDeleted: bool = False):
        params = {
            "moduleName": moduleName,
            "ownerContextId": ownerContextId,
            "ownerContextClassName": ownerContextClassName,
            "showDeleted": showDeleted
        }

        return self._api.get("/api/list", parameters=params)

    def getTree(self, moduleName: str, ownerContextId: int = None,
                ownerContextClassName: str = None, showDeleted: bool = False):
        params = {
            "moduleName": moduleName,
            "ownerContextId": ownerContextId,
            "ownerContextClassName": ownerContextClassName,
            "showDeleted": showDeleted
        }

        return self._api.get("/api/tree", parameters=params)

    def getObject(self, moduleName: str, objectId: int, objectRid: int = None):
        params = {
            "moduleName": moduleName,
            "objectId": objectId,
            "objectRid": objectRid
        }

        return self._api.get("/api/get", parameters=params)

    def getObjectWithSubobjects(self, moduleName: str, objectId: int, objectRid: int = None):
        params = {
            "moduleName": moduleName,
            "objectId": objectId,
            "objectRid": objectRid
        }

        return self._api.get("/api/read", parameters=params)

    def createObject(self, object: QuickRestoObject, moduleName: str, ownerContextId: int = None,
                     ownerContextClassName: str = None, parentContextId: int = None,
                     parentContextClassName: str = None):
        params = {
            "moduleName": moduleName,
            "ownerContextId": ownerContextId,
            "ownerContextClassName": ownerContextClassName,
            "parentContextId": parentContextId,
            "parentContextClassName": parentContextClassName
        }

        json_data = object.get_json_object()

        return self._api.post("/api/create", parameters=params, json_data=json_data)

    def updateObject(self, object: QuickRestoObject, moduleName: str, ownerContextId: int = None,
                     ownerContextClassName: str = None, parentContextId: int = None,
                     parentContextClassName: str = None):
        params = {
            "moduleName": moduleName,
            "ownerContextId": ownerContextId,
            "ownerContextClassName": ownerContextClassName,
            "parentContextId": parentContextId,
            "parentContextClassName": parentContextClassName
        }

        json_data = object.get_json_object()

        return self._api.post("/api/update", parameters=params, json_data=json_data)

    def removeObject(self, object: QuickRestoObject, moduleName: str, ownerContextId: int = None,
                     ownerContextClassName: str = None, parentContextId: int = None,
                     parentContextClassName: str = None):
        params = {
            "moduleName": moduleName,
            "ownerContextId": ownerContextId,
            "ownerContextClassName": ownerContextClassName,
            "parentContextId": parentContextId,
            "parentContextClassName": parentContextClassName
        }

        json_data = object.get_json_object()

        return self._api.post("/api/remove", parameters=params, json_data=json_data)

    def recoverObject(self, object: QuickRestoObject, moduleName: str, ownerContextId: int = None,
                      ownerContextClassName: str = None, parentContextId: int = None,
                      parentContextClassName: str = None):
        params = {
            "moduleName": moduleName,
            "ownerContextId": ownerContextId,
            "ownerContextClassName": ownerContextClassName,
            "parentContextId": parentContextId,
            "parentContextClassName": parentContextClassName
        }

        json_data = object.get_json_object()

        return self._api.post("/api/recover", parameters=params, json_data=json_data)



from datetime import date

from quick_resto_API.quick_resto_objects.modules.front.table import Table
from quick_resto_API.quick_resto_objects.quick_resto_object import QuickRestoObject


class PreorderInfo(QuickRestoObject):
    @property
    def cancellation_reason_comment(self) -> str:
        return self._cancellation_reason_comment

    @property
    def create_date(self) -> date:
        return self._create_date

    @property
    def discount_percent(self) -> int:
        return self._discount_percent

    @property
    def front_total_absolute_charge(self) -> int:
        return self._front_total_absolute_charge

    @property
    def front_total_absolute_discount(self) -> int:
        return self._front_total_absolute_discount

    @property
    def front_total_discount(self) -> int:
        return self._front_total_discount

    @property
    def guest_count(self) -> int:
        return self._guest_count

    @property
    def print_time(self) -> int:
        return self._print_time

    @property
    def sum(self) -> int:
        return self._sum

    @property
    def sum_with_discount(self) -> int:
        return self._sum_with_discount

    @property
    def table(self) -> Table:
        return self._table

    @property
    def table_order_doc_id(self) -> str:
        return self._table_order_doc_id

    def __init__(self, cancellationReasonComment: str = None, createDate: date = None, discountPercent: int = None,
                 frontTotalAbsoluteCharge: int = None,
                 frontTotalAbsoluteDiscount: int = None, frontTotalDiscount: int = None, guestCount: int = None,
                 printTime: int = None, sum: int = None,
                 sumWithDiscount: int = None, table: dict = None, tableOrderDocId: str = None, **kwargs):
        class_name = "ru.edgex.quickresto.modules.front.preorders.PreorderInfo"

        super().__init__(class_name=class_name, **kwargs)

        self._cancellation_reason_comment: str = cancellationReasonComment
        self._create_date = createDate
        self._discount_percent: int = discountPercent
        self._front_total_absolute_charge: int = frontTotalAbsoluteCharge
        self._front_total_absolute_discount: int = frontTotalAbsoluteDiscount
        self._front_total_discount: int = frontTotalDiscount
        self._guest_count: int = guestCount
        self._print_time: int = printTime
        self._sum: int = sum
        self._sum_with_discount: int = sumWithDiscount

        if table is not None: 
            self._table = Table(**table)
        else:
            self._table = None
            
        self._table_order_doc_id: str = tableOrderDocId



from ..core.order_discard_reason import OrderDiscardReason
from quick_resto_API.quick_resto_objects.modules.personnel.employee import Employee
from quick_resto_API.quick_resto_objects.modules.front.table_scheme import TableScheme
from quick_resto_API.quick_resto_objects.modules.warehouse.sale_place import SalePlace
from quick_resto_API.quick_resto_objects.quick_resto_object import QuickRestoObject

class Cancellation(QuickRestoObject):
    @property
    def local_create_time(self) -> str:
        return self._local_create_time

    @property
    def local_time_zone_offset_min(self) -> int:
        return self._local_time_zone_offset_min

    @property
    def create_terminal_sale_place_doc_id(self) -> str:
        return self._create_terminal_sale_place_doc_id

    @property
    def create_terminal_sale_place(self) -> SalePlace:
        return self._create_terminal_sale_place

    @property
    def table_scheme_doc_id(self) -> str:
        return self._table_scheme_doc_id

    @property
    def table_scheme(self) -> TableScheme:
        return self._table_scheme

    @property
    def cancellation_reason_doc_id(self) -> str:
        return self._cancellation_reason_doc_id

    @property
    def cancellation_reason(self) -> OrderDiscardReason:
        return self._cancellation_reason

    @property
    def user_doc_id(self) -> str:
        return self._user_doc_id

    @property
    def employee(self) -> Employee:
        return self._employee

    @property
    def table_order_doc_id(self) -> str:
        return self._table_order_doc_id

    @property
    def comment(self) -> str:
        return self._comment

    @property
    def description(self) -> str:
        return self._description

    @property
    def with_dismission(self) -> bool:
        return self._with_dismission

    def __init__(self, version: int = None, serverRegisterTime: str = None, localCreateTime: str = None, localTimeZoneOffsetMin: int = None, 
                    createTerminalSalePlaceDocId: str = None, createTerminalSalePlace: dict = None, tableSchemeDocId: str = None, tableScheme: dict = None, 
                    cancellationReasonDocId: str = None, cancellationReason: dict = None, userDocId: str = None, employee: dict = None, tableOrderDocId: str = None, 
                    comment: str = None, description: str = None, withDismission: bool = None, **kwargs):
        class_name = "ru.edgex.quickresto.modules.front.cancellations.Cancellation"
        super().__init__(class_name=class_name, **kwargs)

        self._version: int = version
        self._server_register_time: str = serverRegisterTime
        self._local_create_time: str = localCreateTime
        self._local_time_zone_offset_min: int = localTimeZoneOffsetMin
        self._create_terminal_sale_place_doc_id: str = createTerminalSalePlaceDocId

        if createTerminalSalePlace is not None: 
            self._create_terminal_sale_place: SalePlace = SalePlace(**createTerminalSalePlace)
        else:
            self._create_terminal_sale_place = None

        self._table_scheme_doc_id: str = tableSchemeDocId

        if tableScheme is not None: 
            self._table_scheme: TableScheme = TableScheme(**tableScheme)
        else:
            self._table_scheme = None

        self._cancellation_reason_doc_id: str = cancellationReasonDocId

        if cancellationReason is not None: 
            self._cancellation_reason: OrderDiscardReason = OrderDiscardReason(**cancellationReason)
        else:
            self._cancellation_reason = None

        self._user_doc_id: str = userDocId

        if employee is not None: 
            self._employee = Employee(**employee)
        else:
            self._employee = None

        self._table_order_doc_id: str = tableOrderDocId
        self._comment: str = comment
        self._description: str = description
        self._with_dismission: bool = withDismission


from quick_resto_API.quick_resto_objects.quick_resto_object import QuickRestoObject


class RaspberryTerminal(QuickRestoObject):
    @property
    def code1_c(self) -> str:
        return self._code1_c

    @property
    def device_id(self) -> str:
        return self._device_id

    @property
    def mac_address(self) -> str:
        return self._mac_address

    @property
    def manufacturer(self) -> str:
        return self._manufacturer

    @property
    def model(self) -> str:
        return self._model

    @property
    def connected(self) -> bool:
        return self._connected

    @property
    def deleted(self) -> bool:
        return self._deleted

    @property
    def name(self) -> str:
        return self._name

    @property
    def serial_number(self) -> str:
        return self._serial_number

    @property
    def hardware_version(self) -> str:
        return self._hardware_version

    @property
    def last_sync_time(self) -> str:
        return self._last_sync_time

    @property
    def software_version(self) -> str:
        return self._software_version

    @property
    def product(self) -> str:
        return self._product

    @property
    def online(self) -> bool:
        return self._online

    def __init__(self, code1C: str, deviceId: str, macAddress: str, manufacturer: str, model: str, connected: bool,
                 deleted: bool, name: str, serialNumber: str, hardwareVersion: str, lastSyncTime: str,
                 softwareVersion: str, product: str, online: bool, **kwargs):
        class_name = "ru.edgex.quickresto.modules.front.terminals.raspberry.RaspberryTerminal"

        super().__init__(class_name=class_name, **kwargs)

        self._code1_c: str = code1C
        self._device_id: str = deviceId
        self._mac_address: str = macAddress
        self._manufacturer: str = manufacturer
        self._model: str = model
        self._connected: bool = connected
        self._deleted: bool = deleted
        self._name: str = name
        self._serial_number: str = serialNumber
        self._hardware_version: str = hardwareVersion
        self._last_sync_time: str = lastSyncTime
        self._software_version: str = softwareVersion
        self._product: str = product
        self._online: bool = online



from distutils.core import setup

setup(
    name="quick_resto_API",
    version="1.0.20",
    description='Quick Resto API',
    packages=['quick_resto_API', 'quick_resto_API.operations_with_objects.modules', 'quick_resto_API.operations_with_objects', 
                'quick_resto_API.quick_resto_objects.modules.alcohol', 'quick_resto_API.quick_resto_objects.modules.core',
                'quick_resto_API.quick_resto_objects.modules.crm','quick_resto_API.quick_resto_objects.modules.front',
                'quick_resto_API.quick_resto_objects.modules.personnel','quick_resto_API.quick_resto_objects.modules.warehouse',
                'quick_resto_API.quick_resto_objects.platform','quick_resto_API.quick_resto_objects'],

    author_email='sergey.rukin1425@gmail.com',
    author='sergeyrukin',
    )


from quick_resto_API.operations_with_objects.operations_with_objects import OperationsWithObjects
from quick_resto_API.operations_with_objects.system_object import SystemObject
from quick_resto_API.quick_resto_api import QuickRestoApi
from quick_resto_API.quick_resto_objects.modules.warehouse.businessman import Businessman
from quick_resto_API.quick_resto_objects.modules.warehouse.natural_person import NaturalPerson
from quick_resto_API.quick_resto_objects.modules.warehouse.organization import Organization
from quick_resto_API.quick_resto_objects.modules.warehouse.provider_group import ProviderGroup

class ProvidersOperations(SystemObject):
    def __init__(self, api: QuickRestoApi):
        self._operations_with_objects = OperationsWithObjects(api)

        self._module_name:str = "warehouse.providers"

    def get_list_of_provider(self, ownerContextId: int = None, ownerContextClassName: str = None,
                           showDeleted: bool = False) -> list[Businessman|NaturalPerson|Organization|ProviderGroup]:

        json_response = self._operations_with_objects.getList(self._module_name,
                                                              ownerContextId, ownerContextClassName, showDeleted).json()

        result:list[Businessman|NaturalPerson|Organization|ProviderGroup] = list()

        for object in json_response:
            if 'Businessman' in object['className']:
                result.append(Businessman(**object))
            elif 'NaturalPerson' in object['className']:
                result.append(NaturalPerson(**object))
            elif 'Organization' in object['className']:
                result.append(Organization(**object))
            elif 'ProviderGroup' in object['className']:
                result.append(ProviderGroup(**object))

        return result

    def get_tree_of_provider(self, ownerContextId: int = None, ownerContextClassName: str = None,
                           showDeleted: bool = False) -> list[Businessman|NaturalPerson|Organization|ProviderGroup]:

        json_response = self._operations_with_objects.getTree(self._module_name,
                                                              ownerContextId, ownerContextClassName, showDeleted).json()

        result:list[Businessman|NaturalPerson|Organization|ProviderGroup] = list()

        for object in json_response:
            if 'Businessman' in object['className']:
                result.append(Businessman(**object))
            elif 'NaturalPerson' in object['className']:
                result.append(NaturalPerson(**object))
            elif 'Organization' in object['className']:
                result.append(Organization(**object))
            elif 'ProviderGroup' in object['className']:
                result.append(ProviderGroup(**object))

        return result

    def get_provider(self, objectId: int, objectRid: int = None) -> Businessman|NaturalPerson|Organization|ProviderGroup:
        json_response = self._operations_with_objects.getObject(self._module_name, objectId, objectRid).json()

        if 'Businessman' in object['className']:
            return Businessman(**json_response)
        elif 'NaturalPerson' in object['className']:
            return NaturalPerson(**json_response)
        elif 'Organization' in object['className']:
            return Organization(**json_response)
        else:
            return ProviderGroup(**json_response)

    def get_provider_with_subobjects(self, objectId: int, objectRid: int = None) -> Businessman|NaturalPerson|Organization|ProviderGroup:
        json_response = self._operations_with_objects.getObjectWithSubobjects(self._module_name, objectId, objectRid).json()

        if 'Businessman' in object['className']:
            return Businessman(**json_response)
        elif 'NaturalPerson' in object['className']:
            return NaturalPerson(**json_response)
        elif 'Organization' in object['className']:
            return Organization(**json_response)
        else:
            return ProviderGroup(**json_response)

    def create_provider(self, object: Businessman|NaturalPerson|Organization|ProviderGroup,ownerContextId: int = None,
                                                ownerContextClassName: str = None, parentContextId: int = None,
                                                parentContextClassName: str = None) -> Businessman|NaturalPerson|Organization|ProviderGroup:

        json_response = self._operations_with_objects.createObject(object, self._module_name, ownerContextId, 
                                                ownerContextClassName, parentContextId, parentContextClassName).json()

        if 'Businessman' in object['className']:
            return Businessman(**json_response)
        elif 'NaturalPerson' in object['className']:
            return NaturalPerson(**json_response)
        elif 'Organization' in object['className']:
            return Organization(**json_response)
        else:
            return ProviderGroup(**json_response)

    def update_provider(self, object: Businessman|NaturalPerson|Organization|ProviderGroup,ownerContextId: int = None,
                                                ownerContextClassName: str = None, parentContextId: int = None,
                                                parentContextClassName: str = None) -> Businessman|NaturalPerson|Organization|ProviderGroup:

        json_response = self._operations_with_objects.updateObject(object, self._module_name, ownerContextId, 
                                                ownerContextClassName, parentContextId, parentContextClassName).json()

        if 'Businessman' in object['className']:
            return Businessman(**json_response)
        elif 'NaturalPerson' in object['className']:
            return NaturalPerson(**json_response)
        elif 'Organization' in object['className']:
            return Organization(**json_response)
        else:
            return ProviderGroup(**json_response)

    def remove_provider(self, object: Businessman|NaturalPerson|Organization|ProviderGroup,ownerContextId: int = None,
                                                ownerContextClassName: str = None, parentContextId: int = None,
                                                parentContextClassName: str = None) -> Businessman|NaturalPerson|Organization|ProviderGroup:

        json_response = self._operations_with_objects.removeObject(object, self._module_name, ownerContextId, 
                                                ownerContextClassName, parentContextId, parentContextClassName).json()

        if 'Businessman' in object['className']:
            return Businessman(**json_response)
        elif 'NaturalPerson' in object['className']:
            return NaturalPerson(**json_response)
        elif 'Organization' in object['className']:
            return Organization(**json_response)
        else:
            return ProviderGroup(**json_response)

    def recover_provider(self, object: Businessman|NaturalPerson|Organization|ProviderGroup,ownerContextId: int = None,
                                                ownerContextClassName: str = None, parentContextId: int = None,
                                                parentContextClassName: str = None) -> Businessman|NaturalPerson|Organization|ProviderGroup:

        json_response = self._operations_with_objects.recoverObject(object, self._module_name, ownerContextId, 
                                                ownerContextClassName, parentContextId, parentContextClassName).json()

        if 'Businessman' in object['className']:
            return Businessman(**json_response)
        elif 'NaturalPerson' in object['className']:
            return NaturalPerson(**json_response)
        elif 'Organization' in object['className']:
            return Organization(**json_response)
        else:
            return ProviderGroup(**json_response)


from quick_resto_API.quick_resto_objects.quick_resto_object import QuickRestoObject


class Group(QuickRestoObject):
    @property
    def name(self) -> str:
        return self._name

    @property
    def group_id(self) -> int:
        return self._group_id

    @property
    def deleted(self) -> bool:
        return self._deleted

    @property
    def discount_value(self) -> float:
        return self._discount_value

    @property
    def customer_operation_limit(self) -> int:
        return self._customer_operation_limit

    def __init__(self, groupId: int = None, deleted: bool = None, name: str = None, discountValue: float = None,
                 customerOperationLimit: int = None,
                 **kwargs):
        class_name = "ru.edgex.quickresto.modules.crm.customer.group.Group"

        super().__init__(class_name=class_name, **kwargs)
        self._group_id: int = groupId
        self._deleted: bool = deleted
        self._name: str = name
        self._discount_value: float = discountValue
        self._customer_operation_limit: int = customerOperationLimit



from quick_resto_API.quick_resto_objects.quick_resto_object import QuickRestoObject


class DiscardReason(QuickRestoObject):
    @property
    def system_order_discard_reason(self) -> str:
        return self._system_order_discard_reason

    @property
    def version(self) -> int:
        return self._version

    @property
    def ref_id(self) -> str:
        return self._ref_id

    @property
    def withdraw_from_store(self) -> bool:
        return self._withdraw_from_store

    @property
    def use_comment(self) -> bool:
        return self._use_comment

    @property
    def save_to_terminals(self) -> bool:
        return self._save_to_terminals

    @property
    def description(self) -> str:
        return self._description

    def __init__(self, systemOrderDiscardReason: str = None, version: int = None, refId: str = None,
                 withdrawFromStore: bool = None,
                 useComment: bool = None, saveToTerminals: bool = None, description: str = None, **kwargs):
        class_name = "ru.edgex.quickresto.modules.warehouse.documents.discard"

        super().__init__(class_name=class_name, **kwargs)

        self._system_order_discard_reason: str = systemOrderDiscardReason
        self._version: int = version
        self._ref_id: str = refId
        self._withdraw_from_store: bool = withdrawFromStore
        self._use_comment: bool = useComment
        self._save_to_terminals: bool = saveToTerminals
        self._description: str = description



from enum import Enum
from quick_resto_API.quick_resto_objects.modules.front.kkm_terminal import KkmTerminal

from quick_resto_API.quick_resto_objects.quick_resto_object import QuickRestoObject


class ShiftStatus(Enum):
    OPENED = "OPENED"
    CLOSE = "CLOSE"
    NONE = "NONE"


StrToShiftStatus = {
    ShiftStatus.OPENED.value: ShiftStatus.OPENED,
    ShiftStatus.CLOSE.value: ShiftStatus.CLOSE,
}


def convert_str_to_system_Unit(value: str) -> ShiftStatus:
    if value in StrToShiftStatus.keys():
        return StrToShiftStatus[value]

    return ShiftStatus.NONE


class Shift(QuickRestoObject):
    @property
    def cash_out_cheks_count(self) -> int:
        return self._cash_out_cheks_count

    @property
    def cash_in_cheks_count(self) -> int:
        return self._cash_in_cheks_count

    @property
    def close_cash_in_register(self) -> int:
        return self._close_cash_in_register

    @property
    def closer_id(self) -> int:
        return self._closer_id

    @property
    def closed(self) -> int:
        return self._closed

    @property
    def code1_c(self) -> str:
        return self._code1_c

    @property
    def incomplete(self) -> bool:
        return self._incomplete

    @property
    def kkm_terminal(self) -> KkmTerminal:
        return self._kkm_terminal

    @property
    def non_fiscal_total_bonuses(self) -> int:
        return self._non_fiscal_total_bonuses

    @property
    def non_fiscal_total_card(self) -> int:
        return self._non_fiscal_total_card

    @property
    def non_fiscal_total_cash(self) -> int:
        return self._non_fiscal_total_cash

    @property
    def non_fiscal_total_return_bonuses(self) -> int:
        return self._non_fiscal_total_return_bonuses

    @property
    def non_fiscal_total_return_card(self) -> int:
        return self._non_fiscal_total_return_card

    @property
    def non_fiscal_total_return_cash(self) -> int:
        return self._non_fiscal_total_return_cash

    @property
    def open_cash_in_register(self) -> int:
        return self._open_cash_in_register

    @property
    def opened(self) -> int:
        return self._opened

    @property
    def opener_id(self) -> str:
        return self._opener_id

    @property
    def orders_count(self) -> int:
        return self._orders_count

    @property
    def return_orders_count(self) -> int:
        return self._return_orders_count

    @property
    def shift_number(self) -> int:
        return self._shift_number

    @property
    def status(self) -> ShiftStatus:
        return self._status

    @property
    def total_card(self) -> int:
        return self._total_card

    @property
    def total_cash(self) -> int:
        return self._total_cash

    @property
    def total_cash_in(self) -> int:
        return self._total_cash_in

    @property
    def total_cash_in_register(self) -> int:
        return self._total_cash_in_register

    @property
    def total_cash_out(self) -> int:
        return self._total_cash_out

    @property
    def total_return_bonuses(self) -> int:
        return self._total_return_bonuses

    @property
    def total_return_card(self) -> int:
        return self._total_return_card

    @property
    def total_return_cash(self) -> int:
        return self._total_return_cash

    @property
    def write_off_total_bonuses(self) -> int:
        return self._write_off_total_bonuses

    @property
    def write_off_total_card(self) -> int:
        return self._write_off_total_card

    @property
    def write_off_total_cash(self) -> int:
        return self._write_off_total_cash

    @property
    def write_off_total_return_bonuses(self) -> int:
        return self._write_off_total_return_bonuses

    @property
    def write_off_total_return_card(self) -> int:
        return self._write_off_total_return_card

    @property
    def write_off_total_return_cash(self) -> int:
        return self._write_off_total_return_cash

    @property
    def z_report_document_number(self) -> int:
        return self._z_report_document_number

    def __init__(self, cashOutCheksCount: int = None, cashInCheksCount: int = None, closeCashInRegister: int = None,
                 closerId: int = None, closed: int = None,
                 code1C: str = None, incomplete: bool = None, kkmTerminal: dict = None,
                 nonFiscalTotalBonuses: int = None, nonFiscalTotalCard: int = None,
                 nonFiscalTotalCash: int = None, nonFiscalTotalReturnBonuses: int = None,
                 nonFiscalTotalReturnCard: int = None,
                 nonFiscalTotalReturnCash: int = None, openCashInRegister: int = None, opened: int = None,
                 openerId: str = None, ordersCount: int = None,
                 returnOrdersCount: int = None, shiftNumber: int = None, status: str = None, totalCard: int = None,
                 totalCash: int = None, totalCashIn: int = None,
                 totalCashInRegister: int = None, totalCashOut: int = None, totalReturnBonuses: int = None,
                 totalReturnCard: int = None, totalReturnCash: int = None,
                 writeOffTotalBonuses: int = None, writeOffTotalCard: int = None, writeOffTotalCash: int = None,
                 writeOffTotalReturnBonuses: int = None,
                 writeOffTotalReturnCard: int = None, writeOffTotalReturnCash: int = None,
                 zReportDocumentNumber: int = None, **kwargs):
        class_name = "ru.edgex.quickresto.modules.front.zreport.Shift"

        super().__init__(class_name=class_name, **kwargs)

        self._cash_out_cheks_count: int = cashOutCheksCount
        self._cash_in_cheks_count: int = cashInCheksCount
        self._close_cash_in_register: int = closeCashInRegister
        self._closer_id: int = closerId
        self._closed: int = closed
        self._code1_c: str = code1C
        self._incomplete: bool = incomplete

        if kkmTerminal is not None:
            self._kkm_terminal = KkmTerminal(**kkmTerminal)
        else:
            self._kkm_terminal = None

        self._non_fiscal_total_bonuses: int = nonFiscalTotalBonuses
        self._non_fiscal_total_card: int = nonFiscalTotalCard
        self._non_fiscal_total_cash: int = nonFiscalTotalCash
        self._non_fiscal_total_return_bonuses: int = nonFiscalTotalReturnBonuses
        self._non_fiscal_total_return_card: int = nonFiscalTotalReturnCard
        self._non_fiscal_total_return_cash: int = nonFiscalTotalReturnCash
        self._open_cash_in_register: int = openCashInRegister
        self._opened: int = opened
        self._opener_id: str = openerId
        self._orders_count: int = ordersCount
        self._return_orders_count: int = returnOrdersCount
        self._shift_number: int = shiftNumber
        self._status: dict = convert_str_to_system_Unit(status)
        self._total_card: int = totalCard
        self._total_cash: int = totalCash
        self._total_cash_in: int = totalCashIn
        self._total_cash_in_register: int = totalCashInRegister
        self._total_cash_out: int = totalCashOut
        self._total_return_bonuses: int = totalReturnBonuses
        self._total_return_card: int = totalReturnCard
        self._total_return_cash: int = totalReturnCash
        self._write_off_total_bonuses: int = writeOffTotalBonuses
        self._write_off_total_card: int = writeOffTotalCard
        self._write_off_total_cash: int = writeOffTotalCash
        self._write_off_total_return_bonuses: int = writeOffTotalReturnBonuses
        self._write_off_total_return_card: int = writeOffTotalReturnCard
        self._write_off_total_return_cash: int = writeOffTotalReturnCash
        self._z_report_document_number: int = zReportDocumentNumber



from quick_resto_API.quick_resto_interface import QuickRestoInterface

from quick_resto_API.operations_with_objects.modules.account_type_operations import AccountTypeOperations, AccountType
from quick_resto_API.operations_with_objects.modules.alcohol_dictionary_operations import AlcoholDictionaryOperations, AlcoholDictionary
from quick_resto_API.operations_with_objects.modules.alcohol_report_operations import AlcoholReportOperations, AlcoholReport
from quick_resto_API.operations_with_objects.modules.bonus_program_operations import BonusProgramOperations, BonusProgram
from quick_resto_API.operations_with_objects.modules.business_operations import BusinessOperations, Business
from quick_resto_API.operations_with_objects.modules.cancellation_operations import CancellationOperations, Cancellation
from quick_resto_API.operations_with_objects.modules.company_info_operations import CompanyInfoOperations,CompanyInfo
from quick_resto_API.operations_with_objects.modules.cooking_invoice_operations import CookingInvoiceOperations,CookingInvoice
from quick_resto_API.operations_with_objects.modules.cooking_place_operations import CookingPlaceOperations,CookingPlace
from quick_resto_API.operations_with_objects.modules.crm_customer_operations import CrmCustomerOperations,CrmCustomer
from quick_resto_API.operations_with_objects.modules.decomposition_invoice_operations import DecompositionInvoiceOperations,DecompositionInvoice
from quick_resto_API.operations_with_objects.modules.discard_invoice_operations import DiscardInvoiceOperations,DiscardInvoice
from quick_resto_API.operations_with_objects.modules.dish_operations import DishOperations,Dish,DishCategory
from quick_resto_API.operations_with_objects.modules.encashment_operations import EncashmentOperations,Encashment
from quick_resto_API.operations_with_objects.modules.exchange_invoice_operations import ExchangeInvoiceOperations,ExchangeInvoice
from quick_resto_API.operations_with_objects.modules.fixed_discount_operations import FixedDiscountOperations,FixedDiscount
from quick_resto_API.operations_with_objects.modules.incoming_invoice_operations import IncomingInvoiceOperations,IncomingInvoice
from quick_resto_API.operations_with_objects.modules.inventory_document_operations import InventoryDocumentOperations,InventoryDocument
from quick_resto_API.operations_with_objects.modules.kkm_terminal_operations import KkmTerminalOperations,KkmTerminal
from quick_resto_API.operations_with_objects.modules.markup_operations import MarkupOperations,Markup
from quick_resto_API.operations_with_objects.modules.measure_unit_operations import MeasureUnitOperations,MeasureUnit
from quick_resto_API.operations_with_objects.modules.modifier_operations import ModifierOperations,Modifier
from quick_resto_API.operations_with_objects.modules.order_discard_reason_operations import OrderDiscardReasonOperations,OrderDiscardReason
from quick_resto_API.operations_with_objects.modules.order_info_operations import OrderInfoOperations,OrderInfo
from quick_resto_API.operations_with_objects.modules.outgoing_invoice_operations import OutgoingInvoiceOperations,OutgoingInvoice
from quick_resto_API.operations_with_objects.modules.packing_unit_operations import PackingUnitOperations,PackingUnit
from quick_resto_API.operations_with_objects.modules.payment_type_operations import PaymentTypeOperations,PaymentType
from quick_resto_API.operations_with_objects.modules.pos_terminal_operations import PosTerminalOperations,PosTerminal
from quick_resto_API.operations_with_objects.modules.preorder_info_operations import PreorderInfoOperations,PreorderInfo
from quick_resto_API.operations_with_objects.modules.processing_invoice_operations import ProcessingInvoiceOperations,ProcessingInvoice
from quick_resto_API.operations_with_objects.modules.providers_operations import ProvidersOperations,ProviderGroup
from quick_resto_API.operations_with_objects.modules.raspberry_terminal_operations import RaspberryTerminalOperations,RaspberryTerminal
from quick_resto_API.operations_with_objects.modules.sale_place_operations import SalePlaceOperations,SalePlace
from quick_resto_API.operations_with_objects.modules.scheduled_discount_operations import ScheduledDiscountOperations,ScheduledDiscount
from quick_resto_API.operations_with_objects.modules.semi_product_operations import SemiProductOperations,SemiProduct
from quick_resto_API.operations_with_objects.modules.shift_operations import ShiftOperations,Shift
from quick_resto_API.operations_with_objects.modules.single_product_operations import SingleProductOperations,SingleCategory
from quick_resto_API.operations_with_objects.modules.store_item_tag_operations import StoreItemTagOperations,StoreItemTag
from quick_resto_API.operations_with_objects.modules.store_operations import StoreOperations,Store
from quick_resto_API.operations_with_objects.modules.table_scheme_operations import TableSchemeOperations,TableScheme
from quick_resto_API.operations_with_objects.modules.terminal_operations import TerminalOperations,Terminal
from quick_resto_API.operations_with_objects.modules.ticket_device_operations import TicketDeviceOperations,TicketDevice


from enum import Enum

from quick_resto_API.quick_resto_objects.modules.front.table_scheme import TableScheme
from quick_resto_API.quick_resto_objects.modules.front.device_command import DeviceCommand
from quick_resto_API.quick_resto_objects.modules.warehouse.organization import Organization
from quick_resto_API.quick_resto_objects.quick_resto_object import QuickRestoObject


class HardwareState(Enum):
    NEW = "NEW"
    INACTIVE = "INACTIVE"
    ACTIVATING = "ACTIVATING"
    ACTIVE = "ACTIVE"
    DEACTIVATIN = "DEACTIVATING"
    DELETED = "DELETED"
    NONE = "NONE"


StrToHardwareState = {
    HardwareState.NEW.value: HardwareState.NEW,
    HardwareState.INACTIVE.value: HardwareState.INACTIVE,
    HardwareState.ACTIVATING.value: HardwareState.ACTIVATING,
    HardwareState.ACTIVE.value: HardwareState.ACTIVE,
    HardwareState.DEACTIVATIN.value: HardwareState.DEACTIVATIN,
    HardwareState.DELETED.value: HardwareState.DELETED,
}


def convert_str_to_hardware_state(value: str) -> HardwareState:
    if value in StrToHardwareState.keys():
        return StrToHardwareState[value]

    return HardwareState.NONE


class TicketDevice(QuickRestoObject):
    @property
    def code1_c(self) -> str:
        return self._code1_c

    @property
    def mac_address(self) -> str:
        return self._mac_address

    @property
    def manufacturer(self) -> str:
        return self._manufacturer

    @property
    def model(self) -> str:
        return self._model

    @property
    def connected(self) -> bool:
        return self._connected

    @property
    def device_id(self) -> str:
        return self._device_id

    @property
    def deleted(self) -> bool:
        return self._deleted

    @property
    def name(self) -> str:
        return self._name

    @property
    def serial_number(self) -> str:
        return self._serial_number

    @property
    def command(self) -> DeviceCommand:
        return self._command

    @property
    def table_scheme(self) -> TableScheme:
        return self._table_scheme

    @property
    def business(self) -> dict:
        return self._business

    @property
    def state(self) -> HardwareState:
        return self._state

    @property
    def kkm_mode(self) -> bool:
        return self._kkm_mode

    @property
    def symbols_per_line(self) -> int:
        return self._symbols_per_line

    @property
    def organization(self) -> Organization:
        return self._organization

    def __init__(self, code1C: str = None, macAddress: str = None, manufacturer: str = None, model: str = None,
                 connected: bool = None,
                 deviceId: str = None, deleted: bool = None, name: str = None, serialNumber: str = None,
                 command: dict = None, tableScheme: dict = None,
                 business: dict = None, state: str = None, kkmMode: bool = None, symbolsPerLine: int = None,
                 organization: dict = None, **kwargs):
        class_name = "ru.edgex.quickresto.modules.front.terminals.ticketdevices.TicketDevice"

        super().__init__(class_name=class_name, **kwargs)

        self._code1_c: str = code1C
        self._mac_address: str = macAddress
        self._manufacturer: str = manufacturer
        self._model: str = model
        self._connected: bool = connected
        self._device_id: str = deviceId
        self._deleted: bool = deleted
        self._name: str = name
        self._serial_number: str = serialNumber

        if tableScheme is not None: 
            self._table_scheme = TableScheme(**tableScheme)
        else:
            self._table_scheme = None

        if tableScheme is not None: 
            self._table_scheme = TableScheme(**tableScheme)
        else:
            self._table_scheme = None

        self._business: dict = business
        self._state = convert_str_to_hardware_state(state)
        self._kkm_mode: bool = kkmMode
        self._symbols_per_line: int = symbolsPerLine

        if organization is not None: 
            self._organization = Organization(**organization)
        else:
            self._organization = None



from quick_resto_API.quick_resto_objects.modules.personnel.user import User
from quick_resto_API.quick_resto_objects.quick_resto_object import QuickRestoObject


class Employee(QuickRestoObject):
    @property
    def system_employee(self) -> str:
        return self._system_employee

    @property
    def user(self) -> User:
        return self._user

    @property
    def first_name(self) -> str:
        return self._first_name

    @property
    def last_name(self) -> str:
        return self._last_name

    @property
    def blocked(self) -> bool:
        return self._blocked

    @property
    def short_name(self) -> str:
        return self._short_name

    @property
    def full_name(self) -> str:
        return self._full_name

    def __init__(self, systemEmployee: str = None, user: dict = None, firstName: str = None, lastName: str = None,
                 blocked: bool = None,
                 shortName: str = None, fullName: str = None, **kwargs):
        class_name = "ru.edgex.quickresto.modules.personnel.employee.Employee"
        super().__init__(class_name=class_name, **kwargs)

        self._system_employee: str = systemEmployee

        if user is not None:
            self._user = User(**user)
        else:
            self._user = None

        self._first_name: str = firstName
        self._last_name: str = lastName
        self._blocked: bool = blocked
        self._short_name: str = shortName
        self._full_name: str = fullName
